@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-__version__ = "5.3.0"
+__version__ = "5.5.post555"
 
 from .api.opencti_api_client import OpenCTIApiClient
 from .api.opencti_api_connector import OpenCTIApiConnector
@@ -11,8 +11,12 @@ from .connector.opencti_connector_helper import (
 )
 from .entities.opencti_attack_pattern import AttackPattern
 from .entities.opencti_campaign import Campaign
+from .entities.opencti_case import Case
 from .entities.opencti_course_of_action import CourseOfAction
+from .entities.opencti_data_component import DataComponent
+from .entities.opencti_data_source import DataSource
 from .entities.opencti_external_reference import ExternalReference
+from .entities.opencti_grouping import Grouping
 from .entities.opencti_identity import Identity
 from .entities.opencti_incident import Incident
 from .entities.opencti_indicator import Indicator
@@ -53,9 +57,13 @@ from .utils.opencti_stix2_utils import OpenCTIStix2Utils
 __all__ = [
     "AttackPattern",
     "Campaign",
+    "Case",
     "ConnectorType",
     "CourseOfAction",
+    "DataComponent",
+    "DataSource",
     "ExternalReference",
+    "Grouping",
     "Identity",
     "Incident",
     "Indicator",
