@@ -11,12 +11,19 @@ import requests
 import urllib3
 from pythonjsonlogger import jsonlogger
 
+from pycti.api import LOGGER
 from pycti.api.opencti_api_connector import OpenCTIApiConnector
 from pycti.api.opencti_api_work import OpenCTIApiWork
 from pycti.entities.opencti_attack_pattern import AttackPattern
 from pycti.entities.opencti_campaign import Campaign
+from pycti.entities.opencti_case import Case
+from pycti.entities.opencti_channel import Channel
 from pycti.entities.opencti_course_of_action import CourseOfAction
+from pycti.entities.opencti_data_component import DataComponent
+from pycti.entities.opencti_data_source import DataSource
+from pycti.entities.opencti_event import Event
 from pycti.entities.opencti_external_reference import ExternalReference
+from pycti.entities.opencti_grouping import Grouping
 from pycti.entities.opencti_identity import Identity
 from pycti.entities.opencti_incident import Incident
 from pycti.entities.opencti_indicator import Indicator
@@ -24,9 +31,11 @@ from pycti.entities.opencti_infrastructure import Infrastructure
 from pycti.entities.opencti_intrusion_set import IntrusionSet
 from pycti.entities.opencti_kill_chain_phase import KillChainPhase
 from pycti.entities.opencti_label import Label
+from pycti.entities.opencti_language import Language
 from pycti.entities.opencti_location import Location
 from pycti.entities.opencti_malware import Malware
 from pycti.entities.opencti_marking_definition import MarkingDefinition
+from pycti.entities.opencti_narrative import Narrative
 from pycti.entities.opencti_note import Note
 from pycti.entities.opencti_observed_data import ObservedData
 from pycti.entities.opencti_opinion import Opinion
@@ -45,8 +54,10 @@ from pycti.entities.opencti_stix_object_or_stix_relationship import (
 from pycti.entities.opencti_stix_sighting_relationship import StixSightingRelationship
 from pycti.entities.opencti_threat_actor import ThreatActor
 from pycti.entities.opencti_tool import Tool
+from pycti.entities.opencti_vocabulary import Vocabulary
 from pycti.entities.opencti_vulnerability import Vulnerability
 from pycti.utils.opencti_stix2 import OpenCTIStix2
+from pycti.utils.opencti_stix2_utils import OpenCTIStix2Utils
 
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
@@ -114,21 +125,19 @@ class OpenCTIApiClient:
             raise ValueError("A TOKEN must be set")
 
         # Configure logger
-        self.log_level = log_level
-        numeric_level = getattr(logging, self.log_level.upper(), None)
-        if not isinstance(numeric_level, int):
-            raise ValueError("Invalid log level: " + self.log_level)
+        log_level = log_level.upper()
+        LOGGER.setLevel(log_level)
 
         if json_logging:
             log_handler = logging.StreamHandler()
-            log_handler.setLevel(self.log_level.upper())
+            log_handler.setLevel(log_level)
             formatter = CustomJsonFormatter(
                 "%(timestamp)s %(level)s %(name)s %(message)s"
             )
             log_handler.setFormatter(formatter)
-            logging.basicConfig(handlers=[log_handler], level=numeric_level, force=True)
+            logging.basicConfig(handlers=[log_handler], level=log_level, force=True)
         else:
-            logging.basicConfig(level=numeric_level)
+            logging.basicConfig(level=log_level)
 
         # Define API
         self.api_token = token
@@ -142,6 +151,7 @@ class OpenCTIApiClient:
         self.stix2 = OpenCTIStix2(self)
 
         # Define the entities
+        self.vocabulary = Vocabulary(self)
         self.label = Label(self)
         self.marking_definition = MarkingDefinition(self)
         self.external_reference = ExternalReference(self, File)
@@ -157,21 +167,29 @@ class OpenCTIApiClient:
         self.stix_sighting_relationship = StixSightingRelationship(self)
         self.stix_cyber_observable_relationship = StixCyberObservableRelationship(self)
         self.identity = Identity(self)
+        self.event = Event(self)
         self.location = Location(self)
         self.threat_actor = ThreatActor(self)
         self.intrusion_set = IntrusionSet(self)
         self.infrastructure = Infrastructure(self)
         self.campaign = Campaign(self)
+        self.case = Case(self)
         self.incident = Incident(self)
         self.malware = Malware(self)
         self.tool = Tool(self)
+        self.channel = Channel(self)
+        self.narrative = Narrative(self)
+        self.language = Language(self)
         self.vulnerability = Vulnerability(self)
         self.attack_pattern = AttackPattern(self)
         self.course_of_action = CourseOfAction(self)
+        self.data_component = DataComponent(self)
+        self.data_source = DataSource(self)
         self.report = Report(self)
         self.note = Note(self)
         self.observed_data = ObservedData(self)
         self.opinion = Opinion(self)
+        self.grouping = Grouping(self)
         self.indicator = Indicator(self)
 
         # Check if openCTI is available
@@ -252,7 +270,7 @@ class OpenCTIApiClient:
                                 str(file_index),
                                 (
                                     file.name,
-                                    io.BytesIO(file.data.encode()),
+                                    io.BytesIO(file.data.encode("utf-8", "replace")),
                                     file.mime,
                                 ),
                             )
@@ -267,7 +285,11 @@ class OpenCTIApiClient:
                     if isinstance(files.data, str):
                         file_multi = (
                             str(file_index),
-                            (files.name, io.BytesIO(files.data.encode()), files.mime),
+                            (
+                                files.name,
+                                io.BytesIO(files.data.encode("utf-8", "replace")),
+                                files.mime,
+                            ),
                         )
                     else:
                         file_multi = (
@@ -305,19 +327,19 @@ class OpenCTIApiClient:
                     else main_error["message"]
                 )
                 if "data" in main_error and "reason" in main_error["data"]:
-                    logging.error(main_error["data"]["reason"])
+                    LOGGER.error(main_error["data"]["reason"])
                     raise ValueError(
                         {"name": error_name, "message": main_error["data"]["reason"]}
                     )
                 else:
-                    logging.error(main_error["message"])
+                    LOGGER.error(main_error["message"])
                     raise ValueError(
                         {"name": error_name, "message": main_error["message"]}
                     )
             else:
                 return result
         else:
-            logging.info(r.text)
+            LOGGER.info(r.text)
             raise ValueError(r.text)
 
     def fetch_opencti_file(self, fetch_uri, binary=False, serialize=False):
@@ -342,7 +364,6 @@ class OpenCTIApiClient:
 
     def log(self, level, message):
         """log a message with defined log level
-
         :param level: must be a valid logging log level (debug, info, warning, error)
         :type level: str
         :param message: the message to log
@@ -350,13 +371,13 @@ class OpenCTIApiClient:
         """
 
         if level == "debug":
-            logging.debug(message)
+            LOGGER.debug(message)
         elif level == "info":
-            logging.info(message)
+            LOGGER.info(message)
         elif level == "warning":
-            logging.warn(message)
+            LOGGER.warning(message)
         elif level == "error":
-            logging.error(message)
+            LOGGER.error(message)
 
     def health_check(self):
         """submit an example request to the OpenCTI API.
@@ -379,7 +400,7 @@ class OpenCTIApiClient:
         rtype: dict
         """
 
-        logging.info("Getting logs worker config...")
+        LOGGER.info("Getting logs worker config...")
         query = """
             query LogsWorkerConfig {
                 logsWorkerConfig {
@@ -483,6 +504,13 @@ class OpenCTIApiClient:
         :rtype: dict
         """
 
+        # Handle process_multiple_fields specific case
+        attribute = OpenCTIStix2Utils.retrieveClassForMethod(
+            self, data, "entity_type", "process_multiple_fields"
+        )
+        if attribute is not None:
+            data = attribute.process_multiple_fields(data)
+
         if data is None:
             return data
         if "createdBy" in data and data["createdBy"] is not None:
@@ -518,6 +546,9 @@ class OpenCTIApiClient:
         if "opinions" in data:
             data["opinions"] = self.process_multiple(data["opinions"])
             data["opinionsIds"] = self.process_multiple_ids(data["opinions"])
+        if "observedData" in data:
+            data["observedData"] = self.process_multiple(data["observedData"])
+            data["observedDataIds"] = self.process_multiple_ids(data["observedData"])
         if "killChainPhases" in data:
             data["killChainPhases"] = self.process_multiple(data["killChainPhases"])
             data["killChainPhasesIds"] = self.process_multiple_ids(
@@ -549,6 +580,13 @@ class OpenCTIApiClient:
         if "importFiles" in data:
             data["importFiles"] = self.process_multiple(data["importFiles"])
             data["importFilesIds"] = self.process_multiple_ids(data["importFiles"])
+        # See aliases of GraphQL query in stix_core_object method
+        if "name_alt" in data:
+            data["name"] = data["name_alt"]
+            del data["name_alt"]
+        if "content_alt" in data:
+            data["content"] = data["content_alt"]
+            del data["content_alt"]
         return data
 
     def upload_file(self, **kwargs):
@@ -563,7 +601,7 @@ class OpenCTIApiClient:
         data = kwargs.get("data", None)
         mime_type = kwargs.get("mime_type", "text/plain")
         if file_name is not None:
-            self.log("info", "Uploading a file.")
+            LOGGER.info("Uploading a file.")
             query = """
                 mutation UploadImport($file: Upload!) {
                     uploadImport(file: $file) {
@@ -581,17 +619,14 @@ class OpenCTIApiClient:
 
             return self.query(query, {"file": (File(file_name, data, mime_type))})
         else:
-            self.log(
-                "error",
-                "[upload] Missing parameters: file_name or data",
-            )
+            LOGGER.error("[upload] Missing parameter: file_name")
             return None
 
     def upload_pending_file(self, **kwargs):
         """upload a file to OpenCTI API
 
         :param `**kwargs`: arguments for file upload (required: `file_name` and `data`)
-        :return: returns the query respons for the file upload
+        :return: returns the query response for the file upload
         :rtype: dict
         """
 
@@ -601,7 +636,7 @@ class OpenCTIApiClient:
         entity_id = kwargs.get("entity_id", None)
 
         if file_name is not None:
-            self.log("info", "Uploading a file.")
+            LOGGER.info("Uploading a file.")
             query = """
                     mutation UploadPending($file: Upload!, $entityId: String) {
                         uploadPending(file: $file, entityId: $entityId) {
@@ -621,10 +656,7 @@ class OpenCTIApiClient:
                 {"file": (File(file_name, data, mime_type)), "entityId": entity_id},
             )
         else:
-            self.log(
-                "error",
-                "[upload] Missing parameters: file_name or data",
-            )
+            LOGGER.error("[upload] Missing parameter: file_name")
             return None
 
     def get_stix_content(self, id):
@@ -634,7 +666,7 @@ class OpenCTIApiClient:
         rtype: dict
         """
 
-        logging.info("Entity in JSON " + id)
+        LOGGER.info("Entity in JSON %s", id)
         query = """
             query StixQuery($id: String!) {
                 stix(id: $id)
