@@ -1,7 +1,7 @@
 import json
-import logging
 from typing import Any, Dict
 
+from pycti.api import LOGGER
 from pycti.connector.opencti_connector import OpenCTIConnector
 
 
@@ -18,7 +18,7 @@ class OpenCTIApiConnector:
         :rtype: dict
         """
 
-        logging.info("Getting connectors ...")
+        LOGGER.info("Getting connectors ...")
         query = """
             query GetConnectors {
                 connectorsForWorker {
@@ -90,8 +90,10 @@ class OpenCTIApiConnector:
                             pass
                         }
                         listen
+                        listen_routing
                         listen_exchange
                         push
+                        push_routing
                         push_exchange
                     }
                     connector_user_id
