@@ -1,8 +1,10 @@
+import asyncio
 import base64
 import datetime
 import json
 import logging
 import os
+import queue
 import signal
 import ssl
 import sys
@@ -10,13 +12,16 @@ import threading
 import time
 import traceback
 import uuid
+from queue import Queue
 from typing import Callable, Dict, List, Optional, Union
 
 import pika
+from pika.adapters.asyncio_connection import AsyncioConnection
 from pika.exceptions import NackError, UnroutableError
 from sseclient import SSEClient
 
 from pycti.api.opencti_api_client import OpenCTIApiClient
+from pycti.connector import LOGGER
 from pycti.connector.opencti_connector import OpenCTIConnector
 from pycti.utils.opencti_stix2_splitter import OpenCTIStix2Splitter
 
@@ -28,10 +33,12 @@ logging.getLogger("pika").setLevel(logging.ERROR)
 
 def killProgramHook(etype, value, tb):
     traceback.print_exception(etype, value, tb)
-    os.kill(os.getpid(), signal.SIGKILL)
+    os.kill(os.getpid(), signal.SIGTERM)
 
 
-sys.excepthook = killProgramHook
+def start_loop(loop):
+    asyncio.set_event_loop(loop)
+    loop.run_forever()
 
 
 def get_config_variable(
@@ -43,7 +50,7 @@ def get_config_variable(
 ) -> Union[bool, int, None, str]:
     """[summary]
 
-    :param env_var: environnement variable name
+    :param env_var: environment variable name
     :param yaml_path: path to yaml config
     :param config: client config dict, defaults to {}
     :param isNumber: specify if the variable is a number, defaults to False
@@ -66,6 +73,9 @@ def get_config_variable(
     if isNumber:
         return int(result)
 
+    if isinstance(result, str) and len(result) == 0:
+        return default
+
     return result
 
 
@@ -93,7 +103,7 @@ def create_ssl_context() -> ssl.SSLContext:
     return ssl_context
 
 
-class ListenQueue(threading.Thread):
+class ListenQueue:
     """Main class for the ListenQueue used in OpenCTIConnectorHelper
 
     :param helper: instance of a `OpenCTIConnectorHelper` class
@@ -105,7 +115,6 @@ class ListenQueue(threading.Thread):
     """
 
     def __init__(self, helper, config: Dict, callback) -> None:
-        threading.Thread.__init__(self)
         self.pika_credentials = None
         self.pika_parameters = None
         self.pika_connection = None
@@ -113,16 +122,20 @@ class ListenQueue(threading.Thread):
         self.helper = helper
         self.callback = callback
         self.host = config["connection"]["host"]
+        self.vhost = config["connection"]["vhost"]
         self.use_ssl = config["connection"]["use_ssl"]
         self.port = config["connection"]["port"]
         self.user = config["connection"]["user"]
         self.password = config["connection"]["pass"]
         self.queue_name = config["listen"]
-        self.exit_event = threading.Event()
-        self.thread = None
+        self.connector_thread = None
+        self.connector_event_loop = None
+        self.queue_event_loop = asyncio.new_event_loop()
+        asyncio.set_event_loop(self.queue_event_loop)
+        self.run()
 
     # noinspection PyUnusedLocal
-    def _process_message(self, channel, method, properties, body) -> None:
+    async def _process_message(self, channel, method, properties, body) -> None:
         """process a message from the rabbit queue
 
         :param channel: channel instance
@@ -137,26 +150,30 @@ class ListenQueue(threading.Thread):
 
         json_data = json.loads(body)
         channel.basic_ack(delivery_tag=method.delivery_tag)
-        self.thread = threading.Thread(target=self._data_handler, args=[json_data])
-        self.thread.start()
+        message_task = self._data_handler(json_data)
         five_minutes = 60 * 5
         time_wait = 0
-        while self.thread.is_alive():  # Loop while the thread is processing
-            if (
-                self.helper.work_id is not None and time_wait > five_minutes
-            ):  # Ping every 5 minutes
-                self.helper.api.work.ping(self.helper.work_id)
-                time_wait = 0
-            else:
-                time_wait += 1
-            time.sleep(1)
-
-        logging.info(
-            "%s",
-            (
-                f"Message (delivery_tag={method.delivery_tag}) processed"
-                ", thread terminated"
-            ),
+        try:
+            while not message_task.done():  # Loop while the task/thread is processing
+                if (
+                    self.helper.work_id is not None and time_wait > five_minutes
+                ):  # Ping every 5 minutes
+                    self.helper.api.work.ping(self.helper.work_id)
+                    time_wait = 0
+                else:
+                    time_wait += 1
+                await asyncio.sleep(1)
+            self.helper.api.work.to_processed(
+                json_data["internal"]["work_id"], message_task.result()
+            )
+        except Exception as e:  # pylint: disable=broad-except
+            logging.exception("Error in message processing, reporting error to API")
+            self.helper.api.work.to_processed(
+                json_data["internal"]["work_id"], str(e), True
+            )
+        LOGGER.info(
+            "Message (delivery_tag=%s) processed, thread terminated",
+            method.delivery_tag,
         )
 
     def _data_handler(self, json_data) -> None:
@@ -166,53 +183,87 @@ class ListenQueue(threading.Thread):
         self.helper.work_id = work_id
         if applicant_id is not None:
             self.helper.applicant_id = applicant_id
-            self.helper.api.set_applicant_id_header(applicant_id)
+            self.helper.api_impersonate.set_applicant_id_header(applicant_id)
         # Execute the callback
         try:
             self.helper.api.work.to_received(
                 work_id, "Connector ready to process the operation"
             )
-            message = self.callback(json_data["event"])
-            self.helper.api.work.to_processed(work_id, message)
+            if asyncio.iscoroutinefunction(self.callback):
+                message = asyncio.run_coroutine_threadsafe(
+                    self.callback(json_data["event"]), self.connector_event_loop
+                )
+            else:
+                message = asyncio.get_running_loop().run_in_executor(
+                    None, self.callback, json_data["event"]
+                )
+            return message
         except Exception as e:  # pylint: disable=broad-except
-            logging.exception("Error in message processing, reporting error to API")
+            LOGGER.exception("Error in message processing, reporting error to API")
             try:
                 self.helper.api.work.to_processed(work_id, str(e), True)
             except:  # pylint: disable=bare-except
-                logging.error("Failing reporting the processing")
+                LOGGER.error("Failing reporting the processing")
 
     def run(self) -> None:
-        while not self.exit_event.is_set():
+        while True:
             try:
                 # Connect the broker
                 self.pika_credentials = pika.PlainCredentials(self.user, self.password)
                 self.pika_parameters = pika.ConnectionParameters(
                     host=self.host,
                     port=self.port,
-                    virtual_host="/",
+                    virtual_host=self.vhost,
                     credentials=self.pika_credentials,
                     ssl_options=pika.SSLOptions(create_ssl_context(), self.host)
                     if self.use_ssl
                     else None,
                 )
-                self.pika_connection = pika.BlockingConnection(self.pika_parameters)
-                self.channel = self.pika_connection.channel()
-                assert self.channel is not None
-                self.channel.basic_consume(
-                    queue=self.queue_name, on_message_callback=self._process_message
+                if asyncio.iscoroutinefunction(self.callback):
+                    self.connector_event_loop = asyncio.new_event_loop()
+                    self.connector_thread = threading.Thread(
+                        target=lambda: start_loop(self.connector_event_loop)
+                    )
+                    self.connector_thread.start()
+                self.pika_connection = AsyncioConnection(
+                    self.pika_parameters,
+                    on_open_callback=self.on_connection_open,
+                    on_open_error_callback=self.on_connection_open_error,
+                    on_close_callback=self.on_connection_closed,
+                    custom_ioloop=self.queue_event_loop,
                 )
-                self.channel.start_consuming()
-            except (KeyboardInterrupt, SystemExit):
-                self.helper.log_info("Connector stop")
-                sys.exit(0)
-            except Exception as e:  # pylint: disable=broad-except
-                self.helper.log_error(str(e))
+                self.pika_connection.ioloop.run_forever()
+                # If the connection fails, sleep between reconnect attempts
                 time.sleep(10)
+            except (KeyboardInterrupt, SystemExit):
+                LOGGER.info("Connector stop")
+                sys.exit(0)
+            except Exception as err:  # pylint: disable=broad-except
+                LOGGER.error("%s", err)
 
-    def stop(self):
-        self.exit_event.set()
-        if self.thread:
-            self.thread.join()
+    # noinspection PyUnusedLocal
+    def on_connection_open(self, _unused_connection):
+        self.pika_connection.channel(on_open_callback=self.on_channel_open)
+
+    # noinspection PyUnusedLocal
+    def on_connection_open_error(self, _unused_connection, err):
+        LOGGER.info("Unable to connect to the queue. %s", err)
+        self.pika_connection.ioloop.stop()
+
+    # noinspection PyUnusedLocal
+    def on_connection_closed(self, _unused_connection, reason):
+        LOGGER.info("The connection to the queue closed: %s", reason)
+        self.pika_connection.ioloop.stop()
+
+    def on_channel_open(self, channel):
+        self.channel = channel
+        assert self.channel is not None
+        self.channel.basic_consume(
+            queue=self.queue_name,
+            on_message_callback=lambda *args: asyncio.create_task(
+                self._process_message(*args)
+            ),
+        )
 
 
 class PingAlive(threading.Thread):
@@ -238,27 +289,55 @@ class PingAlive(threading.Thread):
                 )
                 if initial_state != remote_state:
                     self.set_state(result["connector_state"])
-                    logging.info(
-                        "%s",
-                        (
-                            "Connector state has been remotely reset to: "
-                            f'"{self.get_state()}"'
-                        ),
+                    LOGGER.info(
+                        'Connector state has been remotely reset to: "%s"',
+                        self.get_state(),
                     )
                 if self.in_error:
                     self.in_error = False
-                    logging.error("API Ping back to normal")
+                    LOGGER.error("API Ping back to normal")
             except Exception:  # pylint: disable=broad-except
                 self.in_error = True
-                logging.error("Error pinging the API")
+                LOGGER.error("Error pinging the API")
             self.exit_event.wait(40)
 
     def run(self) -> None:
-        logging.info("Starting ping alive thread")
+        LOGGER.info("Starting ping alive thread")
         self.ping()
 
     def stop(self) -> None:
-        logging.info("Preparing for clean shutdown")
+        LOGGER.info("Preparing for clean shutdown")
+        self.exit_event.set()
+
+
+class StreamAlive(threading.Thread):
+    def __init__(self, q) -> None:
+        threading.Thread.__init__(self)
+        self.q = q
+        self.exit_event = threading.Event()
+
+    def run(self) -> None:
+        try:
+            LOGGER.info("Starting stream alive thread")
+            time_since_last_heartbeat = 0
+            while not self.exit_event.is_set():
+                time.sleep(5)
+                try:
+                    self.q.get(block=False)
+                    time_since_last_heartbeat = 0
+                except queue.Empty:
+                    time_since_last_heartbeat = time_since_last_heartbeat + 5
+                    if time_since_last_heartbeat > 45:
+                        LOGGER.error(
+                            "Time since last heartbeat exceeded 45s, stopping the connector"
+                        )
+                        break
+            sys.excepthook(*sys.exc_info())
+        except:
+            sys.excepthook(*sys.exc_info())
+
+    def stop(self) -> None:
+        LOGGER.info("Preparing for clean shutdown")
         self.exit_event.set()
 
 
@@ -285,155 +364,106 @@ class ListenStream(threading.Thread):
         self.verify_ssl = verify_ssl
         self.start_timestamp = start_timestamp
         self.live_stream_id = live_stream_id
-        self.listen_delete = listen_delete if listen_delete is not None else True
-        self.no_dependencies = no_dependencies if no_dependencies is not None else False
+        self.listen_delete = listen_delete
+        self.no_dependencies = no_dependencies
         self.recover_iso_date = recover_iso_date
-        self.with_inferences = with_inferences if with_inferences is not None else False
+        self.with_inferences = with_inferences
         self.exit_event = threading.Event()
         self.exit = False
 
     def run(self) -> None:  # pylint: disable=too-many-branches
         try:
             current_state = self.helper.get_state()
+            start_from = self.start_timestamp
+            recover_until = self.recover_iso_date
             if current_state is None:
-                current_state = {
-                    "connectorStartTime": self.helper.date_now_z(),
-                    "connectorLastEventId": f"{self.start_timestamp}-0"
-                    if self.start_timestamp is not None
-                    and len(self.start_timestamp) > 0
-                    else "-",
-                }
-                self.helper.set_state(current_state)
-
-            # If URL and token are provided, likely consuming a remote stream
-            if self.url is not None and self.token is not None:
-                # If a live stream ID, appending the URL
-                if self.live_stream_id is not None:
-                    live_stream_uri = f"/{self.live_stream_id}"
-                elif self.helper.connect_live_stream_id is not None:
-                    live_stream_uri = f"/{self.helper.connect_live_stream_id}"
-                else:
-                    live_stream_uri = ""
-                # Live stream "from" should be empty if start from the beginning
-                if (
-                    self.live_stream_id is not None
-                    or self.helper.connect_live_stream_id is not None
-                ):
-
-                    live_stream_from = (
-                        f"?from={current_state['connectorLastEventId']}"
-                        if "connectorLastEventId" in current_state
-                        and current_state["connectorLastEventId"] != "-"
-                        else "?from=0-0&recover="
-                        + (
-                            current_state["connectorStartTime"]
-                            if self.recover_iso_date is None
-                            else self.recover_iso_date
-                        )
-                    )
-                # Global stream "from" should be 0 if starting from the beginning
-                else:
-                    live_stream_from = "?from=" + (
-                        current_state["connectorLastEventId"]
-                        if "connectorLastEventId" in current_state
-                        and current_state["connectorLastEventId"] != "-"
-                        else "0-0"
-                    )
-                live_stream_url = (
-                    f"{self.url}/stream{live_stream_uri}{live_stream_from}"
-                )
-                opencti_ssl_verify = (
-                    self.verify_ssl if self.verify_ssl is not None else True
-                )
-                logging.info(
-                    "%s",
-                    (
-                        "Starting listening stream events (URL: "
-                        f"{live_stream_url}, SSL verify: {opencti_ssl_verify}, Listen Delete: {self.listen_delete})"
-                    ),
-                )
-                messages = SSEClient(
-                    live_stream_url,
-                    headers={
-                        "authorization": "Bearer " + self.token,
-                        "listen-delete": "false"
-                        if self.listen_delete is False
-                        else "true",
-                        "no-dependencies": "true"
-                        if self.no_dependencies is True
-                        else "false",
-                        "with-inferences": "true"
-                        if self.helper.connect_live_stream_with_inferences is True
-                        else "false",
-                    },
-                    verify=opencti_ssl_verify,
-                )
+                # First run, if no timestamp in config, put "0-0"
+                if start_from is None:
+                    start_from = "0-0"
+                # First run, if no recover iso date in config, put today
+                if recover_until is None:
+                    recover_until = self.helper.date_now_z()
             else:
-                live_stream_uri = (
-                    f"/{self.helper.connect_live_stream_id}"
-                    if self.helper.connect_live_stream_id is not None
-                    else ""
-                )
-                if self.helper.connect_live_stream_id is not None:
-                    live_stream_from = (
-                        f"?from={current_state['connectorLastEventId']}"
-                        if "connectorLastEventId" in current_state
-                        and current_state["connectorLastEventId"] != "-"
-                        else "?from=0-0&recover="
-                        + (
-                            self.helper.date_now_z()
-                            if self.recover_iso_date is None
-                            else self.recover_iso_date
-                        )
-                    )
-                # Global stream "from" should be 0 if starting from the beginning
+                # Get start_from from state
+                # Backward compat
+                if "connectorLastEventId" in current_state:
+                    start_from = current_state["connectorLastEventId"]
+                # Current implem
                 else:
-                    live_stream_from = "?from=" + (
-                        current_state["connectorLastEventId"]
-                        if "connectorLastEventId" in current_state
-                        and current_state["connectorLastEventId"] != "-"
-                        else "0-0"
-                    )
-                live_stream_url = f"{self.helper.opencti_url}/stream{live_stream_uri}{live_stream_from}"
-                logging.info(
-                    "%s",
-                    (
-                        f"Starting listening stream events (URL: {live_stream_url}"
-                        f", SSL verify: {self.helper.opencti_ssl_verify}, Listen Delete: {self.helper.connect_live_stream_listen_delete}, No Dependencies: {self.helper.connect_live_stream_no_dependencies})"
-                    ),
-                )
-                messages = SSEClient(
-                    live_stream_url,
-                    headers={
-                        "authorization": "Bearer " + self.helper.opencti_token,
-                        "listen-delete": "false"
-                        if self.helper.connect_live_stream_listen_delete is False
-                        else "true",
-                        "no-dependencies": "true"
-                        if self.helper.connect_live_stream_no_dependencies is True
-                        else "false",
-                        "with-inferences": "true"
-                        if self.helper.connect_live_stream_with_inferences is True
-                        else "false",
-                    },
-                    verify=self.helper.opencti_ssl_verify,
-                )
+                    start_from = current_state["start_from"]
+                # Get recover_until from state
+                # Backward compat
+                if "connectorStartTime" in current_state:
+                    recover_until = current_state["connectorStartTime"]
+                # Current implem
+                else:
+                    recover_until = current_state["recover_until"]
+            # Set state
+            self.helper.set_state(
+                {"start_from": start_from, "recover_until": recover_until}
+            )
+            # Start the stream alive watchdog
+            q = Queue(maxsize=1)
+            stream_alive = StreamAlive(q)
+            stream_alive.start()
+            # Computing args building
+            live_stream_url = self.url
+            # In case no recover is explicitely set
+            if recover_until is not False and recover_until not in [
+                "no",
+                "none",
+                "No",
+                "None",
+                "false",
+                "False",
+            ]:
+                live_stream_url = live_stream_url + "?recover=" + recover_until
+            listen_delete = str(self.listen_delete).lower()
+            no_dependencies = str(self.no_dependencies).lower()
+            with_inferences = str(self.with_inferences).lower()
+            LOGGER.info(
+                'Starting to listen stream events on "%s" '
+                "(listen-delete: %s, no-dependencies: %s, with-inferences: %s)",
+                *(live_stream_url, listen_delete, no_dependencies, with_inferences),
+            )
+            messages = SSEClient(
+                live_stream_url,
+                start_from,
+                headers={
+                    "authorization": "Bearer " + self.token,
+                    "listen-delete": listen_delete,
+                    "no-dependencies": no_dependencies,
+                    "with-inferences": with_inferences,
+                },
+                verify=self.verify_ssl,
+            )
             # Iter on stream messages
             for msg in messages:
                 if self.exit:
+                    stream_alive.stop()
                     break
-                if msg.event == "heartbeat" or msg.event == "connected":
-                    continue
-                if msg.event == "sync":
-                    if msg.id is not None:
+                if msg.id is not None:
+                    try:
+                        q.put(msg.event, block=False)
+                    except queue.Full:
+                        pass
+                    if msg.event == "heartbeat" or msg.event == "connected":
                         state = self.helper.get_state()
-                        state["connectorLastEventId"] = str(msg.id)
-                        self.helper.set_state(state)
-                else:
-                    self.callback(msg)
-                    if msg.id is not None:
+                        # state can be None if reset from the UI
+                        # In this case, default parameters will be used but SSE Client needs to be restarted
+                        if state is None:
+                            self.exit = True
+                        else:
+                            state["start_from"] = str(msg.id)
+                            self.helper.set_state(state)
+                    else:
+                        self.callback(msg)
                         state = self.helper.get_state()
-                        state["connectorLastEventId"] = str(msg.id)
+                        # state can be None if reset from the UI
+                        # In this case, default parameters will be used but SSE Client needs to be restarted
+                        if state is None:
+                            self.exit = True
+                        state["start_from"] = str(msg.id)
                         self.helper.set_state(state)
         except:
             sys.excepthook(*sys.exc_info())
@@ -451,6 +481,8 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
     """
 
     def __init__(self, config: Dict) -> None:
+        sys.excepthook = killProgramHook
+
         # Load API config
         self.opencti_url = get_config_variable(
             "OPENCTI_URL", ["opencti", "url"], config
@@ -462,7 +494,7 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
             "OPENCTI_SSL_VERIFY", ["opencti", "ssl_verify"], config, False, True
         )
         self.opencti_json_logging = get_config_variable(
-            "OPENCTI_JSON_LOGGING", ["opencti", "json_logging"], config
+            "OPENCTI_JSON_LOGGING", ["opencti", "json_logging"], config, False, True
         )
         # Load connector config
         self.connect_id = get_config_variable(
@@ -499,6 +531,16 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
             False,
             False,
         )
+        self.connect_live_stream_recover_iso_date = get_config_variable(
+            "CONNECTOR_LIVE_STREAM_RECOVER_ISO_DATE",
+            ["connector", "live_stream_recover_iso_date"],
+            config,
+        )
+        self.connect_live_stream_start_timestamp = get_config_variable(
+            "CONNECTOR_LIVE_STREAM_START_TIMESTAMP",
+            ["connector", "live_stream_start_timestamp"],
+            config,
+        )
         self.connect_name = get_config_variable(
             "CONNECTOR_NAME", ["connector", "name"], config
         )
@@ -507,6 +549,7 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
             ["connector", "confidence_level"],
             config,
             True,
+            50,
         )
         self.connect_scope = get_config_variable(
             "CONNECTOR_SCOPE", ["connector", "scope"], config
@@ -522,8 +565,8 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
             False,
         )
         self.log_level = get_config_variable(
-            "CONNECTOR_LOG_LEVEL", ["connector", "log_level"], config
-        )
+            "CONNECTOR_LOG_LEVEL", ["connector", "log_level"], config, default="INFO"
+        ).upper()
         self.connect_run_and_terminate = get_config_variable(
             "CONNECTOR_RUN_AND_TERMINATE",
             ["connector", "run_and_terminate"],
@@ -540,20 +583,25 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
         )
 
         # Configure logger
-        numeric_level = getattr(
-            logging, self.log_level.upper() if self.log_level else "INFO", None
-        )
-        if not isinstance(numeric_level, int):
-            raise ValueError(f"Invalid log level: {self.log_level}")
-        logging.basicConfig(level=numeric_level)
+        logging.basicConfig(level=self.log_level)
 
         # Initialize configuration
+        # - Classic API that will be directly attached to the connector rights
         self.api = OpenCTIApiClient(
             self.opencti_url,
             self.opencti_token,
             self.log_level,
             json_logging=self.opencti_json_logging,
         )
+        # - Impersonate API that will use applicant id
+        # Behave like standard api if applicant not found
+        self.api_impersonate = OpenCTIApiClient(
+            self.opencti_url,
+            self.opencti_token,
+            self.log_level,
+            json_logging=self.opencti_json_logging,
+        )
+
         # Register the connector in OpenCTI
         self.connector = OpenCTIConnector(
             self.connect_id,
@@ -564,10 +612,10 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
             self.connect_only_contextual,
         )
         connector_configuration = self.api.connector.register(self.connector)
-        logging.info("%s", f"Connector registered with ID: {self.connect_id}")
+        LOGGER.info("Connector registered with ID: %s", self.connect_id)
         self.connector_id = connector_configuration["id"]
         self.work_id = None
-        self.applicant_id = connector_configuration["connector_user"]["id"]
+        self.applicant_id = connector_configuration["connector_user_id"]
         self.connector_state = connector_configuration["connector_state"]
         self.config = connector_configuration["config"]
 
@@ -605,10 +653,12 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
         """sets the connector state
 
         :param state: state object
-        :type state: Dict
+        :type state: Dict or None
         """
-
-        self.connector_state = json.dumps(state)
+        if isinstance(state, Dict):
+            self.connector_state = json.dumps(state)
+        else:
+            self.connector_state = None
 
     def get_state(self) -> Optional[Dict]:
         """get the connector state
@@ -639,7 +689,7 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
             if initial_state != remote_state:
                 self.api.connector.ping(self.connector_id, initial_state)
         except Exception:  # pylint: disable=broad-except
-            logging.error("Error pinging the API")
+            LOGGER.error("Error pinging the API")
 
     def listen(self, message_callback: Callable[[Dict], str]) -> None:
         """listen for messages and register callback function
@@ -649,7 +699,6 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
         """
 
         self.listen_queue = ListenQueue(self, self.config, message_callback)
-        self.listen_queue.start()
 
     def listen_stream(
         self,
@@ -659,16 +708,66 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
         verify_ssl=None,
         start_timestamp=None,
         live_stream_id=None,
-        listen_delete=True,
-        no_dependencies=False,
+        listen_delete=None,
+        no_dependencies=None,
         recover_iso_date=None,
-        with_inferences=False,
+        with_inferences=None,
     ) -> ListenStream:
         """listen for messages and register callback function
 
         :param message_callback: callback function to process messages
         """
-
+        # URL
+        if url is None:
+            url = self.opencti_url
+        # Token
+        if token is None:
+            token = self.opencti_token
+        # Verify SSL
+        if verify_ssl is None:
+            verify_ssl = self.opencti_ssl_verify
+        # Live Stream ID
+        if live_stream_id is None and self.connect_live_stream_id is not None:
+            live_stream_id = self.connect_live_stream_id
+        # Listen delete
+        if listen_delete is None and self.connect_live_stream_listen_delete is not None:
+            listen_delete = self.connect_live_stream_listen_delete
+        elif listen_delete is None:
+            listen_delete = False
+        # No deps
+        if (
+            no_dependencies is None
+            and self.connect_live_stream_no_dependencies is not None
+        ):
+            no_dependencies = self.connect_live_stream_no_dependencies
+        elif no_dependencies is None:
+            no_dependencies = False
+        # With inferences
+        if (
+            with_inferences is None
+            and self.connect_live_stream_with_inferences is not None
+        ):
+            with_inferences = self.connect_live_stream_with_inferences
+        elif with_inferences is None:
+            with_inferences = False
+        # Start timestamp
+        if (
+            start_timestamp is None
+            and self.connect_live_stream_start_timestamp is not None
+        ):
+            start_timestamp = str(self.connect_live_stream_start_timestamp) + "-0"
+        elif start_timestamp is not None:
+            start_timestamp = str(start_timestamp) + "-0"
+        # Recover ISO date
+        if (
+            recover_iso_date is None
+            and self.connect_live_stream_recover_iso_date is not None
+        ):
+            recover_iso_date = self.connect_live_stream_recover_iso_date
+        # Generate the stream URL
+        url = url + "/stream"
+        if live_stream_id is not None:
+            url = url + "/" + live_stream_id
         self.listen_stream = ListenStream(
             self,
             message_callback,
@@ -695,16 +794,16 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
         return self.connector
 
     def log_error(self, msg: str) -> None:
-        logging.error(msg)
+        LOGGER.error(msg)
 
     def log_info(self, msg: str) -> None:
-        logging.info(msg)
+        LOGGER.info(msg)
 
     def log_debug(self, msg: str) -> None:
-        logging.debug(msg)
+        LOGGER.debug(msg)
 
     def log_warning(self, msg: str) -> None:
-        logging.warning(msg)
+        LOGGER.warning(msg)
 
     def date_now(self) -> str:
         """get the current date (UTC)
@@ -786,7 +885,7 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
         pika_parameters = pika.ConnectionParameters(
             host=self.config["connection"]["host"],
             port=self.config["connection"]["port"],
-            virtual_host="/",
+            virtual_host=self.config["connection"]["vhost"],
             credentials=pika_credentials,
             ssl_options=pika.SSLOptions(
                 create_ssl_context(), self.config["connection"]["host"]
@@ -841,7 +940,9 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
             "applicant_id": self.applicant_id,
             "action_sequence": sequence,
             "entities_types": entities_types,
-            "content": base64.b64encode(bundle.encode("utf-8")).decode("utf-8"),
+            "content": base64.b64encode(bundle.encode("utf-8", "escape")).decode(
+                "utf-8"
+            ),
             "update": update,
         }
         if work_id is not None:
@@ -849,17 +950,16 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
 
         # Send the message
         try:
-            routing_key = "push_routing_" + self.connector_id
             channel.basic_publish(
                 exchange=self.config["push_exchange"],
-                routing_key=routing_key,
+                routing_key=self.config["push_routing"],
                 body=json.dumps(message),
                 properties=pika.BasicProperties(
                     delivery_mode=2,  # make message persistent
                 ),
             )
         except (UnroutableError, NackError) as e:
-            logging.error("Unable to send bundle, retry...%s", e)
+            LOGGER.error("Unable to send bundle, retry...%s", e)
             self._send_bundle(channel, bundle, **kwargs)
 
     def stix2_get_embedded_objects(self, item) -> Dict:
@@ -1005,10 +1105,25 @@ class OpenCTIConnectorHelper:  # pylint: disable=too-many-public-methods
         """
 
         allowed_tlps: Dict[str, List[str]] = {
-            "TLP:RED": ["TLP:WHITE", "TLP:GREEN", "TLP:AMBER", "TLP:RED"],
-            "TLP:AMBER": ["TLP:WHITE", "TLP:GREEN", "TLP:AMBER"],
-            "TLP:GREEN": ["TLP:WHITE", "TLP:GREEN"],
-            "TLP:WHITE": ["TLP:WHITE"],
+            "TLP:RED": [
+                "TLP:WHITE",
+                "TLP:CLEAR",
+                "TLP:GREEN",
+                "TLP:AMBER",
+                "TLP:AMBER+STRICT",
+                "TLP:RED",
+            ],
+            "TLP:AMBER+STRICT": [
+                "TLP:WHITE",
+                "TLP:CLEAR",
+                "TLP:GREEN",
+                "TLP:AMBER",
+                "TLP:AMBER+STRICT",
+            ],
+            "TLP:AMBER": ["TLP:WHITE", "TLP:CLEAR", "TLP:GREEN", "TLP:AMBER"],
+            "TLP:GREEN": ["TLP:WHITE", "TLP:CLEAR", "TLP:GREEN"],
+            "TLP:WHITE": ["TLP:WHITE", "TLP:CLEAR"],
+            "TLP:CLEAR": ["TLP:WHITE", "TLP:CLEAR"],
         }
 
         return tlp in allowed_tlps[max_tlp]
