@@ -1,6 +1,11 @@
 # coding: utf-8
 
 import json
+import uuid
+
+from stix2.canonicalization.Canonicalize import canonicalize
+
+from pycti.entities import LOGGER
 
 
 class AttackPattern:
@@ -21,6 +26,7 @@ class AttackPattern:
                     entity_type
                     parent_types
                     spec_version
+                    identity_class
                     name
                     description
                     roles
@@ -84,6 +90,19 @@ class AttackPattern:
                         external_id
                         created
                         modified
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
                     }
                 }
             }
@@ -102,7 +121,7 @@ class AttackPattern:
                 edges {
                     node {
                         id
-                        standard_id                            
+                        standard_id
                         entity_type
                         kill_chain_name
                         phase_name
@@ -111,9 +130,33 @@ class AttackPattern:
                         modified
                     }
                 }
-            }        
+            }
+            importFiles {
+                edges {
+                    node {
+                        id
+                        name
+                        size
+                        metaData {
+                            mimetype
+                            version
+                        }
+                    }
+                }
+            }
         """
 
+    @staticmethod
+    def generate_id(name, x_mitre_id=None):
+        name = name.lower().strip()
+        if x_mitre_id is not None:
+            data = {"x_mitre_id": x_mitre_id}
+        else:
+            data = {"name": name}
+        data = canonicalize(data, utf8=False)
+        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
+        return "attack-pattern--" + id
+
     """
         List Attack-Pattern objects
 
@@ -137,9 +180,7 @@ class AttackPattern:
         if get_all:
             first = 500
 
-        self.opencti.log(
-            "info", "Listing Attack-Patterns with filters " + json.dumps(filters) + "."
-        )
+        LOGGER.info("Listing Attack-Patterns with filters %s.", json.dumps(filters))
         query = (
             """
             query AttackPatterns($filters: [AttackPatternsFiltering], $search: String, $first: Int, $after: ID, $orderBy: AttackPatternsOrdering, $orderMode: OrderingMode) {
@@ -179,7 +220,7 @@ class AttackPattern:
             final_data = final_data + data
             while result["data"]["attackPatterns"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["attackPatterns"]["pageInfo"]["endCursor"]
-                self.opencti.log("info", "Listing Attack-Patterns after " + after)
+                LOGGER.info("Listing Attack-Patterns after " + after)
                 result = self.opencti.query(
                     query,
                     {
@@ -201,7 +242,7 @@ class AttackPattern:
 
     """
         Read a Attack-Pattern object
-        
+
         :param id: the id of the Attack-Pattern
         :param filters: the filters to apply if no id provided
         :return Attack-Pattern object
@@ -212,7 +253,7 @@ class AttackPattern:
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            self.opencti.log("info", "Reading Attack-Pattern {" + id + "}.")
+            LOGGER.info("Reading Attack-Pattern {%s}.", id)
             query = (
                 """
                 query AttackPattern($id: String!) {
@@ -237,9 +278,7 @@ class AttackPattern:
             else:
                 return None
         else:
-            self.opencti.log(
-                "error", "[opencti_attack_pattern] Missing parameters: id or filters"
-            )
+            LOGGER.error("[opencti_attack_pattern] Missing parameters: id or filters")
             return None
 
     """
@@ -261,17 +300,19 @@ class AttackPattern:
         created = kwargs.get("created", None)
         modified = kwargs.get("modified", None)
         name = kwargs.get("name", None)
-        description = kwargs.get("description", "")
+        description = kwargs.get("description", None)
         aliases = kwargs.get("aliases", None)
         x_mitre_platforms = kwargs.get("x_mitre_platforms", None)
         x_mitre_permissions_required = kwargs.get("x_mitre_permissions_required", None)
         x_mitre_detection = kwargs.get("x_mitre_detection", None)
         x_mitre_id = kwargs.get("x_mitre_id", None)
         kill_chain_phases = kwargs.get("killChainPhases", None)
+        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        granted_refs = kwargs.get("objectOrganization", None)
         update = kwargs.get("update", False)
 
-        if name is not None and description is not None:
-            self.opencti.log("info", "Creating Attack-Pattern {" + name + "}.")
+        if name is not None:
+            LOGGER.info("Creating Attack-Pattern {%s}.", name)
             query = """
                 mutation AttackPatternAdd($input: AttackPatternAddInput) {
                     attackPatternAdd(input: $input) {
@@ -290,6 +331,7 @@ class AttackPattern:
                         "createdBy": created_by,
                         "objectMarking": object_marking,
                         "objectLabel": object_label,
+                        "objectOrganization": granted_refs,
                         "externalReferences": external_references,
                         "revoked": revoked,
                         "confidence": confidence,
@@ -304,6 +346,7 @@ class AttackPattern:
                         "x_mitre_detection": x_mitre_detection,
                         "x_mitre_id": x_mitre_id,
                         "killChainPhases": kill_chain_phases,
+                        "x_opencti_stix_ids": x_opencti_stix_ids,
                         "update": update,
                     }
                 },
@@ -312,9 +355,8 @@ class AttackPattern:
                 result["data"]["attackPatternAdd"]
             )
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_attack_pattern] Missing parameters: name and description",
+            LOGGER.error(
+                "[opencti_attack_pattern] Missing parameters: name and description"
             )
 
     """
@@ -330,32 +372,66 @@ class AttackPattern:
         update = kwargs.get("update", False)
         if stix_object is not None:
             # Extract external ID
-            x_mitre_id = "Unknown"
+            x_mitre_id = None
             if "x_mitre_id" in stix_object:
                 x_mitre_id = stix_object["x_mitre_id"]
-            if "external_references" in stix_object:
+            elif (
+                self.opencti.get_attribute_in_mitre_extension("id", stix_object)
+                is not None
+            ):
+                x_mitre_id = self.opencti.get_attribute_in_mitre_extension(
+                    "id", stix_object
+                )
+            elif "external_references" in stix_object:
                 for external_reference in stix_object["external_references"]:
                     if (
                         external_reference["source_name"] == "mitre-attack"
                         or external_reference["source_name"] == "mitre-pre-attack"
                         or external_reference["source_name"] == "mitre-mobile-attack"
+                        or external_reference["source_name"] == "mitre-ics-attack"
                         or external_reference["source_name"] == "amitt-attack"
                     ):
-                        x_mitre_id = external_reference["external_id"]
+                        x_mitre_id = (
+                            external_reference["external_id"]
+                            if "external_id" in external_reference
+                            else None
+                        )
 
-            # TODO: Compatibility with OpenCTI 3.X to be REMOVED
+            # Search in extensions
             if "x_opencti_order" not in stix_object:
                 stix_object["x_opencti_order"] = (
-                    stix_object["x_opencti_level"]
-                    if "x_opencti_level" in stix_object
+                    self.opencti.get_attribute_in_extension("order", stix_object)
+                    if self.opencti.get_attribute_in_extension("order", stix_object)
+                    is not None
                     else 0
                 )
-            if "x_mitre_id" not in stix_object:
-                stix_object["x_mitre_id"] = (
-                    stix_object["x_opencti_external_id"]
-                    if "x_opencti_external_id" in stix_object
-                    else None
+            if "x_mitre_platforms" not in stix_object:
+                stix_object[
+                    "x_mitre_platforms"
+                ] = self.opencti.get_attribute_in_mitre_extension(
+                    "platforms", stix_object
                 )
+            if "x_mitre_permissions_required" not in stix_object:
+                stix_object[
+                    "x_mitre_permissions_required"
+                ] = self.opencti.get_attribute_in_mitre_extension(
+                    "permissions_required", stix_object
+                )
+            if "x_mitre_detection" not in stix_object:
+                stix_object[
+                    "x_mitre_detection"
+                ] = self.opencti.get_attribute_in_mitre_extension(
+                    "detection", stix_object
+                )
+            if "x_opencti_stix_ids" not in stix_object:
+                stix_object[
+                    "x_opencti_stix_ids"
+                ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+            if "granted_refs" not in stix_object:
+                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
+                    "granted_refs", stix_object
+                )
+
             return self.create(
                 stix_id=stix_object["id"],
                 createdBy=extras["created_by_id"]
@@ -399,9 +475,29 @@ class AttackPattern:
                 killChainPhases=extras["kill_chain_phases_ids"]
                 if "kill_chain_phases_ids" in extras
                 else None,
+                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
+                if "x_opencti_stix_ids" in stix_object
+                else None,
+                objectOrganization=stix_object["granted_refs"]
+                if "granted_refs" in stix_object
+                else None,
                 update=update,
             )
         else:
-            self.opencti.log(
-                "error", "[opencti_attack_pattern] Missing parameters: stixObject"
-            )
+            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
+
+    def delete(self, **kwargs):
+        id = kwargs.get("id", None)
+        if id is not None:
+            LOGGER.info("Deleting Attack Pattern {%s}.", id)
+            query = """
+                 mutation AttackPatternEdit($id: ID!) {
+                     attackPatternEdit(id: $id) {
+                         delete
+                     }
+                 }
+             """
+            self.opencti.query(query, {"id": id})
+        else:
+            LOGGER.error("[attack_pattern] Missing parameters: id")
+            return None
