@@ -5,6 +5,8 @@ import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
+from pycti.entities import LOGGER
+
 
 class Campaign:
     def __init__(self, opencti):
@@ -160,9 +162,7 @@ class Campaign:
         if get_all:
             first = 500
 
-        self.opencti.log(
-            "info", "Listing Campaigns with filters " + json.dumps(filters) + "."
-        )
+        LOGGER.info("Listing Campaigns with filters %s.", json.dumps(filters))
         query = (
             """
             query Campaigns($filters: [CampaignsFiltering], $search: String, $first: Int, $after: ID, $orderBy: CampaignsOrdering, $orderMode: OrderingMode) {
@@ -213,7 +213,7 @@ class Campaign:
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            self.opencti.log("info", "Reading Campaign {" + id + "}.")
+            LOGGER.info("Reading Campaign {%s}.", id)
             query = (
                 """
                 query Campaign($id: String!) {
@@ -238,9 +238,7 @@ class Campaign:
             else:
                 return None
         else:
-            self.opencti.log(
-                "error", "[opencti_campaign] Missing parameters: id or filters"
-            )
+            LOGGER.error("[opencti_campaign] Missing parameters: id or filters")
             return None
 
     """
@@ -267,11 +265,12 @@ class Campaign:
         first_seen = kwargs.get("first_seen", None)
         last_seen = kwargs.get("last_seen", None)
         objective = kwargs.get("objective", None)
-        update = kwargs.get("update", False)
+        granted_refs = kwargs.get("objectOrganization", None)
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        update = kwargs.get("update", False)
 
         if name is not None and description is not None:
-            self.opencti.log("info", "Creating Campaign {" + name + "}.")
+            LOGGER.info("Creating Campaign {%s}.", name)
             query = """
                 mutation CampaignAdd($input: CampaignAddInput) {
                     campaignAdd(input: $input) {
@@ -289,6 +288,7 @@ class Campaign:
                         "stix_id": stix_id,
                         "createdBy": created_by,
                         "objectMarking": object_marking,
+                        "objectOrganization": granted_refs,
                         "objectLabel": object_label,
                         "externalReferences": external_references,
                         "revoked": revoked,
@@ -309,9 +309,7 @@ class Campaign:
             )
             return self.opencti.process_multiple_fields(result["data"]["campaignAdd"])
         else:
-            self.opencti.log(
-                "error", "[opencti_campaign] Missing parameters: name and description"
-            )
+            LOGGER.error("[opencti_campaign] Missing parameters: name and description")
 
     """
         Import a Campaign object from a STIX2 object
@@ -325,12 +323,15 @@ class Campaign:
         extras = kwargs.get("extras", {})
         update = kwargs.get("update", False)
         if stix_object is not None:
-
             # Search in extensions
             if "x_opencti_stix_ids" not in stix_object:
                 stix_object[
                     "x_opencti_stix_ids"
                 ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+            if "granted_refs" not in stix_object:
+                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
+                    "granted_refs", stix_object
+                )
 
             return self.create(
                 stix_id=stix_object["id"],
@@ -372,9 +373,10 @@ class Campaign:
                 x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
                 if "x_opencti_stix_ids" in stix_object
                 else None,
+                objectOrganization=stix_object["granted_refs"]
+                if "granted_refs" in stix_object
+                else None,
                 update=update,
             )
         else:
-            self.opencti.log(
-                "error", "[opencti_campaign] Missing parameters: stixObject"
-            )
+            LOGGER.error("[opencti_campaign] Missing parameters: stixObject")
