@@ -1,6 +1,11 @@
 # coding: utf-8
 
 import json
+import uuid
+
+from stix2.canonicalization.Canonicalize import canonicalize
+
+from pycti.entities import LOGGER
 
 
 class CourseOfAction:
@@ -21,6 +26,7 @@ class CourseOfAction:
                     entity_type
                     parent_types
                     spec_version
+                    identity_class
                     name
                     description
                     roles
@@ -36,7 +42,7 @@ class CourseOfAction:
                                 color
                             }
                         }
-                    }                    
+                    }
                 }
                 ... on Organization {
                     x_opencti_organization_type
@@ -84,6 +90,19 @@ class CourseOfAction:
                         external_id
                         created
                         modified
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
                     }
                 }
             }
@@ -95,8 +114,32 @@ class CourseOfAction:
             description
             x_opencti_aliases
             x_mitre_id
+            importFiles {
+                edges {
+                    node {
+                        id
+                        name
+                        size
+                        metaData {
+                            mimetype
+                            version
+                        }
+                    }
+                }
+            }
         """
 
+    @staticmethod
+    def generate_id(name, x_mitre_id=None):
+        name = name.lower().strip()
+        if x_mitre_id is not None:
+            data = {"x_mitre_id": x_mitre_id}
+        else:
+            data = {"name": name}
+        data = canonicalize(data, utf8=False)
+        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
+        return "course-of-action--" + id
+
     """
         List Course-Of-Action objects
 
@@ -120,10 +163,7 @@ class CourseOfAction:
         if get_all:
             first = 500
 
-        self.opencti.log(
-            "info",
-            "Listing Course-Of-Actions with filters " + json.dumps(filters) + ".",
-        )
+        LOGGER.info("Listing Course-Of-Actions with filters %s.", json.dumps(filters))
         query = (
             """
             query CoursesOfAction($filters: [CoursesOfActionFiltering], $search: String, $first: Int, $after: ID, $orderBy: CoursesOfActionOrdering, $orderMode: OrderingMode) {
@@ -163,7 +203,7 @@ class CourseOfAction:
 
     """
         Read a Course-Of-Action object
-        
+
         :param id: the id of the Course-Of-Action
         :param filters: the filters to apply if no id provided
         :return Course-Of-Action object
@@ -174,7 +214,7 @@ class CourseOfAction:
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            self.opencti.log("info", "Reading Course-Of-Action {" + id + "}.")
+            LOGGER.info("Reading Course-Of-Action {%s}.", id)
             query = (
                 """
                 query CourseOfAction($id: String!) {
@@ -201,9 +241,7 @@ class CourseOfAction:
             else:
                 return None
         else:
-            self.opencti.log(
-                "error", "[opencti_course_of_action] Missing parameters: id or filters"
-            )
+            LOGGER.error("[opencti_course_of_action] Missing parameters: id or filters")
             return None
 
     """
@@ -227,11 +265,13 @@ class CourseOfAction:
         name = kwargs.get("name", None)
         description = kwargs.get("description", "")
         x_opencti_aliases = kwargs.get("x_opencti_aliases", None)
+        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
         x_mitre_id = kwargs.get("x_mitre_id", None)
+        granted_refs = kwargs.get("objectOrganization", None)
         update = kwargs.get("update", False)
 
         if name is not None and description is not None:
-            self.opencti.log("info", "Creating Course Of Action {" + name + "}.")
+            LOGGER.info("Creating Course Of Action {%s}.", name)
             query = """
                 mutation CourseOfActionAdd($input: CourseOfActionAddInput) {
                     courseOfActionAdd(input: $input) {
@@ -250,6 +290,7 @@ class CourseOfAction:
                         "createdBy": created_by,
                         "objectMarking": object_marking,
                         "objectLabel": object_label,
+                        "objectOrganization": granted_refs,
                         "externalReferences": external_references,
                         "revoked": revoked,
                         "confidence": confidence,
@@ -259,6 +300,7 @@ class CourseOfAction:
                         "name": name,
                         "description": description,
                         "x_opencti_aliases": x_opencti_aliases,
+                        "x_opencti_stix_ids": x_opencti_stix_ids,
                         "x_mitre_id": x_mitre_id,
                         "update": update,
                     }
@@ -268,9 +310,8 @@ class CourseOfAction:
                 result["data"]["courseOfActionAdd"]
             )
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_course_of_action] Missing parameters: name and description",
+            LOGGER.error(
+                "[opencti_course_of_action] Missing parameters: name and description"
             )
 
     """
@@ -289,7 +330,14 @@ class CourseOfAction:
             x_mitre_id = None
             if "x_mitre_id" in stix_object:
                 x_mitre_id = stix_object["x_mitre_id"]
-            if "external_references" in stix_object:
+            elif (
+                self.opencti.get_attribute_in_mitre_extension("id", stix_object)
+                is not None
+            ):
+                x_mitre_id = self.opencti.get_attribute_in_mitre_extension(
+                    "id", stix_object
+                )
+            elif "external_references" in stix_object:
                 for external_reference in stix_object["external_references"]:
                     if (
                         external_reference["source_name"] == "mitre-attack"
@@ -299,6 +347,20 @@ class CourseOfAction:
                     ):
                         x_mitre_id = external_reference["external_id"]
 
+            # Search in extensions
+            if "x_opencti_aliases" not in stix_object:
+                stix_object[
+                    "x_opencti_aliases"
+                ] = self.opencti.get_attribute_in_extension("aliases", stix_object)
+            if "x_opencti_stix_ids" not in stix_object:
+                stix_object[
+                    "x_opencti_stix_ids"
+                ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+            if "granted_refs" not in stix_object:
+                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
+                    "granted_refs", stix_object
+                )
+
             return self.create(
                 stix_id=stix_object["id"],
                 createdBy=extras["created_by_id"]
@@ -326,11 +388,15 @@ class CourseOfAction:
                 )
                 if "description" in stix_object
                 else "",
+                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
+                if "x_opencti_stix_ids" in stix_object
+                else None,
                 x_opencti_aliases=self.opencti.stix2.pick_aliases(stix_object),
                 x_mitre_id=x_mitre_id,
+                objectOrganization=stix_object["granted_refs"]
+                if "granted_refs" in stix_object
+                else None,
                 update=update,
             )
         else:
-            self.opencti.log(
-                "error", "[opencti_course_of_action] Missing parameters: stixObject"
-            )
+            LOGGER.error("[opencti_course_of_action] Missing parameters: stixObject")
