@@ -5,6 +5,8 @@ import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
+from pycti.entities import LOGGER
+
 
 class Indicator:
     """Main Indicator class for OpenCTI
@@ -202,9 +204,7 @@ class Indicator:
         if get_all:
             first = 100
 
-        self.opencti.log(
-            "info", "Listing Indicators with filters " + json.dumps(filters) + "."
-        )
+        LOGGER.info("Listing Indicators with filters %s.", json.dumps(filters))
         query = (
             """
                 query Indicators($filters: [IndicatorsFiltering], $search: String, $first: Int, $after: ID, $orderBy: IndicatorsOrdering, $orderMode: OrderingMode) {
@@ -244,7 +244,7 @@ class Indicator:
             final_data = final_data + data
             while result["data"]["indicators"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["indicators"]["pageInfo"]["endCursor"]
-                self.opencti.log("info", "Listing Indicators after " + after)
+                LOGGER.info("Listing Indicators after " + after)
                 result = self.opencti.query(
                     query,
                     {
@@ -285,7 +285,7 @@ class Indicator:
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            self.opencti.log("info", "Reading Indicator {" + id + "}.")
+            LOGGER.info("Reading Indicator {%s}.", id)
             query = (
                 """
                     query Indicator($id: String!) {
@@ -310,9 +310,7 @@ class Indicator:
             else:
                 return None
         else:
-            self.opencti.log(
-                "error", "[opencti_indicator] Missing parameters: id or filters"
-            )
+            LOGGER.error("[opencti_indicator] Missing parameters: id or filters")
             return None
 
     def create(self, **kwargs):
@@ -353,6 +351,7 @@ class Indicator:
         kill_chain_phases = kwargs.get("killChainPhases", None)
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
         create_observables = kwargs.get("x_opencti_create_observables", False)
+        granted_refs = kwargs.get("objectOrganization", None)
         update = kwargs.get("update", False)
 
         if (
@@ -362,7 +361,7 @@ class Indicator:
         ):
             if x_opencti_main_observable_type == "File":
                 x_opencti_main_observable_type = "StixFile"
-            self.opencti.log("info", "Creating Indicator {" + name + "}.")
+            LOGGER.info("Creating Indicator {%s}.", name)
             query = """
                 mutation IndicatorAdd($input: IndicatorAddInput) {
                     indicatorAdd(input: $input) {
@@ -392,6 +391,7 @@ class Indicator:
                         "createdBy": created_by,
                         "objectMarking": object_marking,
                         "objectLabel": object_label,
+                        "objectOrganization": granted_refs,
                         "externalReferences": external_references,
                         "revoked": revoked,
                         "confidence": confidence,
@@ -419,9 +419,9 @@ class Indicator:
             )
             return self.opencti.process_multiple_fields(result["data"]["indicatorAdd"])
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_indicator] Missing parameters: name or pattern or x_opencti_main_observable_type",
+            LOGGER.error(
+                "[opencti_indicator] Missing parameters: "
+                "name or pattern or x_opencti_main_observable_type"
             )
 
     def add_stix_cyber_observable(self, **kwargs):
@@ -441,21 +441,17 @@ class Indicator:
             if indicator is None:
                 indicator = self.read(id=id)
             if indicator is None:
-                self.opencti.log(
-                    "error",
-                    "[opencti_indicator] Cannot add Object Ref, indicator not found",
+                LOGGER.error(
+                    "[opencti_indicator] Cannot add Object Ref, indicator not found"
                 )
                 return False
             if stix_cyber_observable_id in indicator["observablesIds"]:
                 return True
             else:
-                self.opencti.log(
-                    "info",
-                    "Adding Stix-Observable {"
-                    + stix_cyber_observable_id
-                    + "} to Indicator {"
-                    + id
-                    + "}",
+                LOGGER.info(
+                    "Adding Stix-Observable {%s} to Indicator {%s}",
+                    stix_cyber_observable_id,
+                    id,
                 )
                 query = """
                     mutation StixCoreRelationshipAdd($input: StixCoreRelationshipAddInput!) {
@@ -477,9 +473,8 @@ class Indicator:
                 )
                 return True
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_indicator] Missing parameters: id and stix cyber_observable_id",
+            LOGGER.error(
+                "[opencti_indicator] Missing parameters: id and stix cyber_observable_id"
             )
             return False
 
@@ -498,7 +493,6 @@ class Indicator:
         extras = kwargs.get("extras", {})
         update = kwargs.get("update", False)
         if stix_object is not None:
-
             # Search in extensions
             if "x_opencti_score" not in stix_object:
                 stix_object[
@@ -530,6 +524,10 @@ class Indicator:
                 stix_object[
                     "x_opencti_stix_ids"
                 ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+            if "granted_refs" not in stix_object:
+                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
+                    "granted_refs", stix_object
+                )
 
             return self.create(
                 stix_id=stix_object["id"],
@@ -582,6 +580,9 @@ class Indicator:
                 x_opencti_detection=stix_object["x_opencti_detection"]
                 if "x_opencti_detection" in stix_object
                 else False,
+                x_mitre_platforms=stix_object["x_mitre_platforms"]
+                if "x_mitre_platforms" in stix_object
+                else None,
                 x_opencti_main_observable_type=stix_object[
                     "x_opencti_main_observable_type"
                 ]
@@ -596,9 +597,10 @@ class Indicator:
                 x_opencti_create_observables=stix_object["x_opencti_create_observables"]
                 if "x_opencti_create_observables" in stix_object
                 else False,
+                objectOrganization=stix_object["granted_refs"]
+                if "granted_refs" in stix_object
+                else None,
                 update=update,
             )
         else:
-            self.opencti.log(
-                "error", "[opencti_attack_pattern] Missing parameters: stixObject"
-            )
+            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
