@@ -1,6 +1,11 @@
 # coding: utf-8
 
 import json
+import uuid
+
+from stix2.canonicalization.Canonicalize import canonicalize
+
+from pycti.entities import LOGGER
 
 
 class KillChainPhase:
@@ -20,6 +25,13 @@ class KillChainPhase:
             updated_at
         """
 
+    @staticmethod
+    def generate_id(phase_name, kill_chain_name):
+        data = {"phase_name": phase_name, "kill_chain_name": kill_chain_name}
+        data = canonicalize(data, utf8=False)
+        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
+        return "kill-chain-phase--" + id
+
     """
         List Kill-Chain-Phase objects
 
@@ -35,13 +47,14 @@ class KillChainPhase:
         after = kwargs.get("after", None)
         order_by = kwargs.get("orderBy", None)
         order_mode = kwargs.get("orderMode", None)
+        custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
         if get_all:
             first = 500
 
-        self.opencti.log(
-            "info", "Listing Kill-Chain-Phase with filters " + json.dumps(filters) + "."
+        LOGGER.info(
+            "Listing Kill-Chain-Phase with filters " + json.dumps(filters) + "."
         )
         query = (
             """
@@ -50,7 +63,7 @@ class KillChainPhase:
                     edges {
                         node {
                             """
-            + self.properties
+            + (custom_attributes if custom_attributes is not None else self.properties)
             + """
                         }
                     }
@@ -91,7 +104,7 @@ class KillChainPhase:
         id = kwargs.get("id", None)
         filters = kwargs.get("filters", None)
         if id is not None:
-            self.opencti.log("info", "Reading Kill-Chain-Phase {" + id + "}.")
+            LOGGER.info("Reading Kill-Chain-Phase {%s}.", id)
             query = (
                 """
                 query KillChainPhase($id: String!) {
@@ -114,9 +127,7 @@ class KillChainPhase:
             else:
                 return None
         else:
-            self.opencti.log(
-                "error", "[opencti_kill_chain_phase] Missing parameters: id or filters"
-            )
+            LOGGER.error("[opencti_kill_chain_phase] Missing parameters: id or filters")
             return None
 
     """
@@ -133,9 +144,10 @@ class KillChainPhase:
         kill_chain_name = kwargs.get("kill_chain_name", None)
         phase_name = kwargs.get("phase_name", None)
         x_opencti_order = kwargs.get("x_opencti_order", 0)
+        update = kwargs.get("update", False)
 
         if kill_chain_name is not None and phase_name is not None:
-            self.opencti.log("info", "Creating Kill-Chain-Phase {" + phase_name + "}.")
+            LOGGER.info("Creating Kill-Chain-Phase {%s}.", phase_name)
             query = (
                 """
                 mutation KillChainPhaseAdd($input: KillChainPhaseAddInput) {
@@ -157,6 +169,7 @@ class KillChainPhase:
                         "kill_chain_name": kill_chain_name,
                         "phase_name": phase_name,
                         "x_opencti_order": x_opencti_order,
+                        "update": update,
                     }
                 },
             )
@@ -164,7 +177,62 @@ class KillChainPhase:
                 result["data"]["killChainPhaseAdd"]
             )
         else:
-            self.opencti.log(
-                "error",
+            LOGGER.error(
                 "[opencti_kill_chain_phase] Missing parameters: kill_chain_name and phase_name",
             )
+
+    """
+        Update a Kill chain object field
+
+        :param id: the Kill chain id
+        :param input: the input of the field
+        :return The updated Kill chain object
+    """
+
+    def update_field(self, **kwargs):
+        id = kwargs.get("id", None)
+        input = kwargs.get("input", None)
+        if id is not None and input is not None:
+            LOGGER.info("Updating Kill chain {%s}.", id)
+            query = """
+                    mutation KillChainPhaseEdit($id: ID!, $input: [EditInput]!) {
+                        killChainPhaseEdit(id: $id) {
+                            fieldPatch(input: $input) {
+                                id
+                                standard_id
+                                entity_type
+                            }
+                        }
+                    }
+                """
+            result = self.opencti.query(
+                query,
+                {
+                    "id": id,
+                    "input": input,
+                },
+            )
+            return self.opencti.process_multiple_fields(
+                result["data"]["killChainPhaseEdit"]["fieldPatch"]
+            )
+        else:
+            LOGGER.error(
+                "[opencti_kill_chain] Missing parameters: id and key and value"
+            )
+            return None
+
+    def delete(self, **kwargs):
+        id = kwargs.get("id", None)
+        if id is not None:
+            LOGGER.info("Deleting Kill-Chain-Phase {%s}.", id)
+            query = """
+                 mutation KillChainPhaseEdit($id: ID!) {
+                     killChainPhaseEdit(id: $id) {
+                         delete
+                     }
+                 }
+             """
+            self.opencti.query(query, {"id": id})
+        else:
+            LOGGER.error("[opencti_kill_chain_phase] Missing parameters: id")
+            return None
