@@ -5,6 +5,8 @@ import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
+from pycti.entities import LOGGER
+
 
 class Malware:
     def __init__(self, opencti):
@@ -178,9 +180,7 @@ class Malware:
         if get_all:
             first = 500
 
-        self.opencti.log(
-            "info", "Listing Malwares with filters " + json.dumps(filters) + "."
-        )
+        LOGGER.info("Listing Malwares with filters %s.", json.dumps(filters))
         query = (
             """
             query Malwares($filters: [MalwaresFiltering], $search: String, $first: Int, $after: ID, $orderBy: MalwaresOrdering, $orderMode: OrderingMode) {
@@ -221,7 +221,7 @@ class Malware:
             final_data = final_data + data
             while result["data"]["malwares"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["malwares"]["pageInfo"]["endCursor"]
-                self.opencti.log("info", "Listing Malwares after " + after)
+                LOGGER.info("Listing Malwares after %s", after)
                 result = self.opencti.query(
                     query,
                     {
@@ -254,7 +254,7 @@ class Malware:
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            self.opencti.log("info", "Reading Malware {" + id + "}.")
+            LOGGER.info("Reading Malware {%s}.", id)
             query = (
                 """
                 query Malware($id: String!) {
@@ -279,9 +279,7 @@ class Malware:
             else:
                 return None
         else:
-            self.opencti.log(
-                "error", "[opencti_malware] Missing parameters: id or filters"
-            )
+            LOGGER.error("[opencti_malware] Missing parameters: id or filters")
             return None
 
     """
@@ -314,10 +312,11 @@ class Malware:
         capabilities = kwargs.get("capabilities", None)
         kill_chain_phases = kwargs.get("killChainPhases", None)
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        granted_refs = kwargs.get("objectOrganization", None)
         update = kwargs.get("update", False)
 
         if name is not None and description is not None:
-            self.opencti.log("info", "Creating Malware {" + name + "}.")
+            LOGGER.info("Creating Malware {%s}.", name)
             query = """
                 mutation MalwareAdd($input: MalwareAddInput) {
                     malwareAdd(input: $input) {
@@ -336,6 +335,7 @@ class Malware:
                         "createdBy": created_by,
                         "objectMarking": object_marking,
                         "objectLabel": object_label,
+                        "objectOrganization": granted_refs,
                         "externalReferences": external_references,
                         "revoked": revoked,
                         "confidence": confidence,
@@ -360,9 +360,7 @@ class Malware:
             )
             return self.opencti.process_multiple_fields(result["data"]["malwareAdd"])
         else:
-            self.opencti.log(
-                "error", "[opencti_malware] Missing parameters: name and description"
-            )
+            LOGGER.error("[opencti_malware] Missing parameters: name and description")
 
     """
         Import an Malware object from a STIX2 object
@@ -376,12 +374,15 @@ class Malware:
         extras = kwargs.get("extras", {})
         update = kwargs.get("update", False)
         if stix_object is not None:
-
             # Search in extensions
             if "x_opencti_stix_ids" not in stix_object:
                 stix_object[
                     "x_opencti_stix_ids"
                 ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+            if "granted_refs" not in stix_object:
+                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
+                    "granted_refs", stix_object
+                )
 
             return self.create(
                 stix_id=stix_object["id"],
@@ -438,9 +439,10 @@ class Malware:
                 x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
                 if "x_opencti_stix_ids" in stix_object
                 else None,
+                objectOrganization=stix_object["granted_refs"]
+                if "granted_refs" in stix_object
+                else None,
                 update=update,
             )
         else:
-            self.opencti.log(
-                "error", "[opencti_attack_pattern] Missing parameters: stixObject"
-            )
+            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
