@@ -5,6 +5,8 @@ import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
+from pycti.entities import LOGGER
+
 
 class MarkingDefinition:
     def __init__(self, opencti):
@@ -52,10 +54,7 @@ class MarkingDefinition:
         if get_all:
             first = 500
 
-        self.opencti.log(
-            "info",
-            "Listing Marking-Definitions with filters " + json.dumps(filters) + ".",
-        )
+        LOGGER.info("Listing Marking-Definitions with filters %s.", json.dumps(filters))
         query = (
             """
             query MarkingDefinitions($filters: [MarkingDefinitionsFiltering], $first: Int, $after: ID, $orderBy: MarkingDefinitionsOrdering, $orderMode: OrderingMode) {
@@ -104,7 +103,7 @@ class MarkingDefinition:
         id = kwargs.get("id", None)
         filters = kwargs.get("filters", None)
         if id is not None:
-            self.opencti.log("info", "Reading Marking-Definition {" + id + "}.")
+            LOGGER.info("Reading Marking-Definition {%s}.", id)
             query = (
                 """
                 query MarkingDefinition($id: String!) {
@@ -127,9 +126,8 @@ class MarkingDefinition:
             else:
                 return None
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_marking_definition] Missing parameters: id or filters",
+            LOGGER.error(
+                "[opencti_marking_definition] Missing parameters: id or filters"
             )
             return None
 
@@ -184,8 +182,7 @@ class MarkingDefinition:
                 result["data"]["markingDefinitionAdd"]
             )
         else:
-            self.opencti.log(
-                "error",
+            LOGGER.error(
                 "[opencti_marking_definition] Missing parameters: definition and definition_type",
             )
 
@@ -201,7 +198,7 @@ class MarkingDefinition:
         id = kwargs.get("id", None)
         input = kwargs.get("input", None)
         if id is not None and input is not None:
-            self.opencti.log("info", "Updating Marking Definition {" + id + "}")
+            LOGGER.info("Updating Marking Definition {%s}.", id)
             query = """
                     mutation MarkingDefinitionEdit($id: ID!, $input: [EditInput]!) {
                         markingDefinitionEdit(id: $id) {
@@ -224,9 +221,8 @@ class MarkingDefinition:
                 result["data"]["markingDefinitionEdit"]["fieldPatch"]
             )
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_marking_definition] Missing parameters: id and key and value",
+            LOGGER.error(
+                "[opencti_marking_definition] Missing parameters: id and key and value"
             )
             return None
 
@@ -260,6 +256,10 @@ class MarkingDefinition:
                 elif "name" in stix_object:
                     definition = stix_object["name"]
 
+            # Replace TLP:WHITE
+            if definition == "TLP:WHITE":
+                definition = "TLP:CLEAR"
+
             # Search in extensions
             if (
                 "x_opencti_order" not in stix_object
@@ -296,14 +296,12 @@ class MarkingDefinition:
                 update=update,
             )
         else:
-            self.opencti.log(
-                "error", "[opencti_marking_definition] Missing parameters: stixObject"
-            )
+            LOGGER.error("[opencti_marking_definition] Missing parameters: stixObject")
 
     def delete(self, **kwargs):
         id = kwargs.get("id", None)
         if id is not None:
-            self.opencti.log("info", "Deleting Marking-Definition {" + id + "}.")
+            LOGGER.info("Deleting Marking-Definition {%s}.", id)
             query = """
                  mutation MarkingDefinitionEdit($id: ID!) {
                      markingDefinitionEdit(id: $id) {
@@ -313,7 +311,5 @@ class MarkingDefinition:
              """
             self.opencti.query(query, {"id": id})
         else:
-            self.opencti.log(
-                "error", "[opencti_marking_definition] Missing parameters: id"
-            )
+            LOGGER.error("[opencti_marking_definition] Missing parameters: id")
             return None
