@@ -1,8 +1,13 @@
 # coding: utf-8
 
+import datetime
 import json
 import uuid
 
+from stix2.canonicalization.Canonicalize import canonicalize
+
+from pycti.entities import LOGGER
+
 
 class Note:
     def __init__(self, opencti):
@@ -109,6 +114,8 @@ class Note:
             attribute_abstract
             content
             authors
+            note_types
+            likelihood
             objects {
                 edges {
                     node {
@@ -185,6 +192,27 @@ class Note:
                         ... on Incident {
                             name
                         }
+                        ... on Event {
+                            name
+                        }
+                        ... on Channel {
+                            name
+                        }
+                        ... on Narrative {
+                            name
+                        }
+                        ... on Language {
+                            name
+                        }
+                        ... on DataComponent {
+                            name
+                        }
+                        ... on DataSource {
+                            name
+                        }
+                        ... on Case {
+                            name
+                        }
                         ... on StixCoreRelationship {
                             standard_id
                             spec_version
@@ -211,8 +239,14 @@ class Note:
         """
 
     @staticmethod
-    def generate_id():
-        return "note--" + str(uuid.uuid4())
+    def generate_id(created, content):
+        content = content.lower().strip()
+        if isinstance(created, datetime.datetime):
+            created = created.isoformat()
+        data = {"content": content, "created": created}
+        data = canonicalize(data, utf8=False)
+        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
+        return "note--" + id
 
     """
         List Note objects
@@ -237,9 +271,7 @@ class Note:
         if get_all:
             first = 100
 
-        self.opencti.log(
-            "info", "Listing Notes with filters " + json.dumps(filters) + "."
-        )
+        LOGGER.info("Listing Notes with filters %s.", json.dumps(filters))
         query = (
             """
             query Notes($filters: [NotesFiltering], $search: String, $first: Int, $after: ID, $orderBy: NotesOrdering, $orderMode: OrderingMode) {
@@ -279,7 +311,7 @@ class Note:
             final_data = final_data + data
             while result["data"]["notes"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["notes"]["pageInfo"]["endCursor"]
-                self.opencti.log("info", "Listing Notes after " + after)
+                LOGGER.info("Listing Notes after " + after)
                 result = self.opencti.query(
                     query,
                     {
@@ -312,7 +344,7 @@ class Note:
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            self.opencti.log("info", "Reading Note {" + id + "}.")
+            LOGGER.info("Reading Note {%s}.", id)
             query = (
                 """
                 query Note($id: String!) {
@@ -349,13 +381,9 @@ class Note:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            self.opencti.log(
-                "info",
-                "Checking StixObjectOrStixRelationship {"
-                + stix_object_or_stix_relationship_id
-                + "} in Note {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Checking StixObjectOrStixRelationship {%s} in {%s}",
+                *(stix_object_or_stix_relationship_id, Note),
             )
             query = """
                 query NoteContainsStixObjectOrStixRelationship($id: String!, $stixObjectOrStixRelationshipId: String!) {
@@ -371,10 +399,7 @@ class Note:
             )
             return result["data"]["noteContainsStixObjectOrStixRelationship"]
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_note] Missing parameters: id or entity_id",
-            )
+            LOGGER.error("[opencti_note] Missing parameters: id or entity_id")
 
     """
         Create a Note object
@@ -398,11 +423,14 @@ class Note:
         abstract = kwargs.get("abstract", None)
         content = kwargs.get("content", None)
         authors = kwargs.get("authors", None)
-        update = kwargs.get("update", False)
+        note_types = kwargs.get("note_types", None)
+        likelihood = kwargs.get("likelihood", None)
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        granted_refs = kwargs.get("objectOrganization", None)
+        update = kwargs.get("update", False)
 
         if content is not None:
-            self.opencti.log("info", "Creating Note {" + content + "}.")
+            LOGGER.info("Creating Note {%s}.", content)
             query = """
                 mutation NoteAdd($input: NoteAddInput) {
                     noteAdd(input: $input) {
@@ -421,6 +449,7 @@ class Note:
                         "createdBy": created_by,
                         "objectMarking": object_marking,
                         "objectLabel": object_label,
+                        "objectOrganization": granted_refs,
                         "objects": objects,
                         "externalReferences": external_references,
                         "revoked": revoked,
@@ -431,6 +460,8 @@ class Note:
                         "attribute_abstract": abstract,
                         "content": content,
                         "authors": authors,
+                        "note_types": note_types,
+                        "likelihood": likelihood,
                         "x_opencti_stix_ids": x_opencti_stix_ids,
                         "update": update,
                     }
@@ -438,10 +469,7 @@ class Note:
             )
             return self.opencti.process_multiple_fields(result["data"]["noteAdd"])
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_note] Missing parameters: content",
-            )
+            LOGGER.error("[opencti_note] Missing parameters: content")
 
     """
         Add a Stix-Entity object to Note object (object_refs)
@@ -462,13 +490,9 @@ class Note:
                 stixObjectOrStixRelationshipId=stix_object_or_stix_relationship_id,
             ):
                 return True
-            self.opencti.log(
-                "info",
-                "Adding StixObjectOrStixRelationship {"
-                + stix_object_or_stix_relationship_id
-                + "} to Note {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Adding StixObjectOrStixRelationship {%s} to Note {%s}",
+                *(stix_object_or_stix_relationship_id, id),
             )
             query = """
                mutation NoteEdit($id: ID!, $input: StixMetaRelationshipAddInput) {
@@ -491,8 +515,7 @@ class Note:
             )
             return True
         else:
-            self.opencti.log(
-                "error",
+            LOGGER.error(
                 "[opencti_note] Missing parameters: id and stix_object_or_stix_relationship_id",
             )
             return False
@@ -511,16 +534,12 @@ class Note:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            self.opencti.log(
-                "info",
-                "Removing StixObjectOrStixRelationship {"
-                + stix_object_or_stix_relationship_id
-                + "} to Note {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Removing StixObjectOrStixRelationship {%s} to {%s}",
+                *(stix_object_or_stix_relationship_id, Note),
             )
             query = """
-               mutation NotetEditRelationDelete($id: ID!, $toId: String!, $relationship_type: String!) {
+               mutation NotetEditRelationDelete($id: ID!, $toId: StixRef!, $relationship_type: String!) {
                    noteEdit(id: $id) {
                         relationDelete(toId: $toId, relationship_type: $relationship_type) {
                             id
@@ -538,9 +557,7 @@ class Note:
             )
             return True
         else:
-            self.opencti.log(
-                "error", "[opencti_note] Missing parameters: id and entity_id"
-            )
+            LOGGER.error("[opencti_note] Missing parameters: id and entity_id")
             return False
 
     """
@@ -555,12 +572,15 @@ class Note:
         extras = kwargs.get("extras", {})
         update = kwargs.get("update", False)
         if stix_object is not None:
-
             # Search in extensions
             if "x_opencti_stix_ids" not in stix_object:
                 stix_object[
                     "x_opencti_stix_ids"
                 ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+            if "granted_refs" not in stix_object:
+                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
+                    "granted_refs", stix_object
+                )
 
             return self.create(
                 stix_id=stix_object["id"],
@@ -594,9 +614,16 @@ class Note:
                 if "x_opencti_stix_ids" in stix_object
                 else None,
                 authors=stix_object["authors"] if "authors" in stix_object else None,
+                note_types=stix_object["note_types"]
+                if "note_types" in stix_object
+                else None,
+                likelihood=stix_object["likelihood"]
+                if "likelihood" in stix_object
+                else None,
+                objectOrganization=stix_object["granted_refs"]
+                if "granted_refs" in stix_object
+                else None,
                 update=update,
             )
         else:
-            self.opencti.log(
-                "error", "[opencti_attack_pattern] Missing parameters: stixObject"
-            )
+            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
