@@ -5,6 +5,8 @@ import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
+from pycti.entities import LOGGER
+
 
 class ObservedData:
     def __init__(self, opencti):
@@ -187,6 +189,27 @@ class ObservedData:
                         ... on Incident {
                             name
                         }
+                        ... on Event {
+                            name
+                        }
+                        ... on Channel {
+                            name
+                        }
+                        ... on Narrative {
+                            name
+                        }
+                        ... on Language {
+                            name
+                        }
+                        ... on DataComponent {
+                            name
+                        }
+                        ... on DataSource {
+                            name
+                        }
+                        ... on Case {
+                            name
+                        }                             
                         ... on StixCoreRelationship {
                             standard_id
                             spec_version
@@ -241,9 +264,7 @@ class ObservedData:
         if get_all:
             first = 500
 
-        self.opencti.log(
-            "info", "Listing ObservedDatas with filters " + json.dumps(filters) + "."
-        )
+        LOGGER.info("Listing ObservedDatas with filters %s.", json.dumps(filters))
         query = (
             """
             query ObservedDatas($filters: [ObservedDatasFiltering], $search: String, $first: Int, $after: ID, $orderBy: ObservedDatasOrdering, $orderMode: OrderingMode) {
@@ -294,7 +315,7 @@ class ObservedData:
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            self.opencti.log("info", "Reading ObservedData {" + id + "}.")
+            LOGGER.info("Reading ObservedData {%s}.", id)
             query = (
                 """
                 query ObservedData($id: String!) {
@@ -331,13 +352,9 @@ class ObservedData:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            self.opencti.log(
-                "info",
-                "Checking StixObjectOrStixRelationship {"
-                + stix_object_or_stix_relationship_id
-                + "} in ObservedData {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Checking StixObjectOrStixRelationship {%s} in ObservedData {%s}",
+                *(stix_object_or_stix_relationship_id, id),
             )
             query = """
                 query ObservedDataContainsStixObjectOrStixRelationship($id: String!, $stixObjectOrStixRelationshipId: String!) {
@@ -353,10 +370,7 @@ class ObservedData:
             )
             return result["data"]["observedDataContainsStixObjectOrStixRelationship"]
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_observedData] Missing parameters: id or entity_id",
-            )
+            LOGGER.error("[opencti_observedData] Missing parameters: id or entity_id")
 
     """
         Create a ObservedData object
@@ -381,6 +395,7 @@ class ObservedData:
         last_observed = kwargs.get("last_observed", None)
         number_observed = kwargs.get("number_observed", None)
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        granted_refs = kwargs.get("objectOrganization", None)
         update = kwargs.get("update", False)
 
         if (
@@ -388,7 +403,7 @@ class ObservedData:
             and last_observed is not None
             and objects is not None
         ):
-            self.opencti.log("info", "Creating ObservedData.")
+            LOGGER.info("Creating ObservedData.")
             query = """
                 mutation ObservedDataAdd($input: ObservedDataAddInput) {
                     observedDataAdd(input: $input) {
@@ -407,6 +422,7 @@ class ObservedData:
                         "createdBy": created_by,
                         "objectMarking": object_marking,
                         "objectLabel": object_label,
+                        "objectOrganization": granted_refs,
                         "objects": objects,
                         "externalReferences": external_references,
                         "revoked": revoked,
@@ -426,9 +442,9 @@ class ObservedData:
                 result["data"]["observedDataAdd"]
             )
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_observedData] Missing parameters: first_observed, last_observed or objects",
+            LOGGER.error(
+                "[opencti_observedData] Missing parameters: "
+                "first_observed, last_observed or objects"
             )
 
     """
@@ -450,13 +466,9 @@ class ObservedData:
                 stixObjectOrStixRelationshipId=stix_object_or_stix_relationship_id,
             ):
                 return True
-            self.opencti.log(
-                "info",
-                "Adding StixObjectOrStixRelationship {"
-                + stix_object_or_stix_relationship_id
-                + "} to ObservedData {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Adding StixObjectOrStixRelationship {%s} to ObservedData {%s}",
+                *(stix_object_or_stix_relationship_id, id),
             )
             query = """
                mutation ObservedDataEdit($id: ID!, $input: StixMetaRelationshipAddInput) {
@@ -479,9 +491,9 @@ class ObservedData:
             )
             return True
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_observedData] Missing parameters: id and stix_object_or_stix_relationship_id",
+            LOGGER.error(
+                "[opencti_observedData] Missing parameters: "
+                "id and stix_object_or_stix_relationship_id"
             )
             return False
 
@@ -499,16 +511,12 @@ class ObservedData:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            self.opencti.log(
-                "info",
-                "Removing StixObjectOrStixRelationship {"
-                + stix_object_or_stix_relationship_id
-                + "} to Observed-Data {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Removing StixObjectOrStixRelationship {%s} to Observed-Data {%s}",
+                *(stix_object_or_stix_relationship_id, id),
             )
             query = """
-               mutation ObservedDataEditRelationDelete($id: ID!, $toId: String!, $relationship_type: String!) {
+               mutation ObservedDataEditRelationDelete($id: ID!, $toId: StixRef!, $relationship_type: String!) {
                    observedDataEdit(id: $id) {
                         relationDelete(toId: $toId, relationship_type: $relationship_type) {
                             id
@@ -526,9 +534,7 @@ class ObservedData:
             )
             return True
         else:
-            self.opencti.log(
-                "error", "[opencti_observed_data] Missing parameters: id and entity_id"
-            )
+            LOGGER.error("[opencti_observed_data] Missing parameters: id and entity_id")
             return False
 
     """
@@ -559,18 +565,24 @@ class ObservedData:
                         objectLabel=extras["object_label_ids"]
                         if "object_label_ids" in extras
                         else [],
+                        objectOrganization=extras["granted_refs_ids"]
+                        if "granted_refs_ids" in extras
+                        else None,
                     )
                 )
                 for item in stix_observable_results:
                     object_refs.append(item["standard_id"])
 
         if stix_object is not None:
-
             # Search in extensions
             if "x_opencti_stix_ids" not in stix_object:
                 stix_object[
                     "x_opencti_stix_ids"
                 ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+            if "granted_refs" not in stix_object:
+                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
+                    "granted_refs", stix_object
+                )
 
             observed_data_result = self.create(
                 stix_id=stix_object["id"],
@@ -606,11 +618,12 @@ class ObservedData:
                 x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
                 if "x_opencti_stix_ids" in stix_object
                 else None,
+                objectOrganization=stix_object["granted_refs"]
+                if "granted_refs" in stix_object
+                else None,
                 update=update,
             )
 
             return observed_data_result
         else:
-            self.opencti.log(
-                "error", "[opencti_attack_pattern] Missing parameters: stixObject"
-            )
+            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
