@@ -3,6 +3,8 @@
 import json
 import uuid
 
+from pycti.entities import LOGGER
+
 
 class Opinion:
     def __init__(self, opencti):
@@ -185,6 +187,9 @@ class Opinion:
                         ... on Incident {
                             name
                         }
+                        ... on Case {
+                            name
+                        }                        
                         ... on StixCoreRelationship {
                             standard_id
                             spec_version
@@ -237,9 +242,7 @@ class Opinion:
         if get_all:
             first = 100
 
-        self.opencti.log(
-            "info", "Listing Opinions with filters " + json.dumps(filters) + "."
-        )
+        LOGGER.info("Listing Opinions with filters %s.", json.dumps(filters))
         query = (
             """
             query Opinions($filters: [OpinionsFiltering], $search: String, $first: Int, $after: ID, $orderBy: OpinionsOrdering, $orderMode: OrderingMode) {
@@ -279,7 +282,7 @@ class Opinion:
             final_data = final_data + data
             while result["data"]["opinions"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["opinions"]["pageInfo"]["endCursor"]
-                self.opencti.log("info", "Listing Opinions after " + after)
+                LOGGER.info("Listing Opinions after " + after)
                 result = self.opencti.query(
                     query,
                     {
@@ -312,7 +315,7 @@ class Opinion:
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            self.opencti.log("info", "Reading Opinion {" + id + "}.")
+            LOGGER.info("Reading Opinion {%s}.", id)
             query = (
                 """
                 query Opinion($id: String!) {
@@ -349,13 +352,9 @@ class Opinion:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            self.opencti.log(
-                "info",
-                "Checking StixObjectOrStixRelationship {"
-                + stix_object_or_stix_relationship_id
-                + "} in Opinion {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Checking StixObjectOrStixRelationship {%s} in Opinion {%s}",
+                *(stix_object_or_stix_relationship_id, id),
             )
             query = """
                 query OpinionContainsStixObjectOrStixRelationship($id: String!, $stixObjectOrStixRelationshipId: String!) {
@@ -371,10 +370,7 @@ class Opinion:
             )
             return result["data"]["opinionContainsStixObjectOrStixRelationship"]
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_opinion] Missing parameters: id or entity_id",
-            )
+            LOGGER.error("[opencti_opinion] Missing parameters: id or entity_id")
 
     """
         Create a Opinion object
@@ -399,10 +395,11 @@ class Opinion:
         authors = kwargs.get("authors", None)
         opinion = kwargs.get("opinion", None)
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        granted_refs = kwargs.get("objectOrganization", None)
         update = kwargs.get("update", False)
 
         if opinion is not None:
-            self.opencti.log("info", "Creating Opinion {" + opinion + "}.")
+            LOGGER.info("Creating Opinion {%s}.", opinion)
             query = """
                 mutation OpinionAdd($input: OpinionAddInput) {
                     opinionAdd(input: $input) {
@@ -421,6 +418,7 @@ class Opinion:
                         "createdBy": created_by,
                         "objectMarking": object_marking,
                         "objectLabel": object_label,
+                        "objectOrganization": granted_refs,
                         "objects": objects,
                         "externalReferences": external_references,
                         "revoked": revoked,
@@ -438,10 +436,7 @@ class Opinion:
             )
             return self.opencti.process_multiple_fields(result["data"]["opinionAdd"])
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_opinion] Missing parameters: content",
-            )
+            LOGGER.error("[opencti_opinion] Missing parameters: content")
 
     """
         Add a Stix-Entity object to Opinion object (object_refs)
@@ -462,13 +457,9 @@ class Opinion:
                 stixObjectOrStixRelationshipId=stix_object_or_stix_relationship_id,
             ):
                 return True
-            self.opencti.log(
-                "info",
-                "Adding StixObjectOrStixRelationship {"
-                + stix_object_or_stix_relationship_id
-                + "} to Opinion {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Adding StixObjectOrStixRelationship {%s} to Opinion {%s}",
+                *(stix_object_or_stix_relationship_id, id),
             )
             query = """
                mutation OpinionEdit($id: ID!, $input: StixMetaRelationshipAddInput) {
@@ -491,8 +482,7 @@ class Opinion:
             )
             return True
         else:
-            self.opencti.log(
-                "error",
+            LOGGER.error(
                 "[opencti_opinion] Missing parameters: id and stix_object_or_stix_relationship_id",
             )
             return False
@@ -511,16 +501,12 @@ class Opinion:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            self.opencti.log(
-                "info",
-                "Removing StixObjectOrStixRelationship {"
-                + stix_object_or_stix_relationship_id
-                + "} to Opinion {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Removing StixObjectOrStixRelationship {%s} to Opinion {%s}",
+                *(stix_object_or_stix_relationship_id, id),
             )
             query = """
-               mutation OpinionEditRelationDelete($id: ID!, $toId: String!, $relationship_type: String!) {
+               mutation OpinionEditRelationDelete($id: ID!, $toId: StixRef!, $relationship_type: String!) {
                    opinionEdit(id: $id) {
                         relationDelete(toId: $toId, relationship_type: $relationship_type) {
                             id
@@ -538,9 +524,7 @@ class Opinion:
             )
             return True
         else:
-            self.opencti.log(
-                "error", "[opencti_opinion] Missing parameters: id and entity_id"
-            )
+            LOGGER.error("[opencti_opinion] Missing parameters: id and entity_id")
             return False
 
     """
@@ -555,12 +539,15 @@ class Opinion:
         extras = kwargs.get("extras", {})
         update = kwargs.get("update", False)
         if stix_object is not None:
-
             # Search in extensions
             if "x_opencti_stix_ids" not in stix_object:
                 stix_object[
                     "x_opencti_stix_ids"
                 ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+            if "granted_refs" not in stix_object:
+                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
+                    "granted_refs", stix_object
+                )
 
             return self.create(
                 stix_id=stix_object["id"],
@@ -596,9 +583,10 @@ class Opinion:
                 if "x_opencti_stix_ids" in stix_object
                 else None,
                 opinion=stix_object["opinion"] if "opinion" in stix_object else None,
+                objectOrganization=stix_object["granted_refs"]
+                if "granted_refs" in stix_object
+                else None,
                 update=update,
             )
         else:
-            self.opencti.log(
-                "error", "[opencti_attack_pattern] Missing parameters: stixObject"
-            )
+            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
