@@ -1,8 +1,13 @@
 # coding: utf-8
 
+import datetime
 import json
+import uuid
 
 from dateutil.parser import parse
+from stix2.canonicalization.Canonicalize import canonicalize
+
+from pycti.entities import LOGGER
 
 
 class Report:
@@ -39,7 +44,7 @@ class Report:
                                 color
                             }
                         }
-                    }                    
+                    }
                 }
                 ... on Organization {
                     x_opencti_organization_type
@@ -186,7 +191,28 @@ class Report:
                         }
                         ... on Incident {
                             name
-                        }                
+                        }
+                        ... on Event {
+                            name
+                        }
+                        ... on Channel {
+                            name
+                        }
+                        ... on Narrative {
+                            name
+                        }
+                        ... on Language {
+                            name
+                        }
+                        ... on DataComponent {
+                            name
+                        }
+                        ... on DataSource {
+                            name
+                        }
+                        ... on Case {
+                            name
+                        }
                         ... on StixCoreRelationship {
                             standard_id
                             spec_version
@@ -194,6 +220,12 @@ class Report:
                             updated_at
                             relationship_type
                         }
+                       ... on StixSightingRelationship {
+                            standard_id
+                            spec_version
+                            created_at
+                            updated_at
+                        }
                     }
                 }
             }
@@ -212,6 +244,16 @@ class Report:
             }
         """
 
+    @staticmethod
+    def generate_id(name, published):
+        name = name.lower().strip()
+        if isinstance(published, datetime.datetime):
+            published = published.isoformat()
+        data = {"name": name, "published": published}
+        data = canonicalize(data, utf8=False)
+        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
+        return "report--" + id
+
     """
         List Report objects
 
@@ -235,9 +277,7 @@ class Report:
         if get_all:
             first = 100
 
-        self.opencti.log(
-            "info", "Listing Reports with filters " + json.dumps(filters) + "."
-        )
+        LOGGER.info("Listing Reports with filters %s.", json.dumps(filters))
         query = (
             """
             query Reports($filters: [ReportsFiltering], $search: String, $first: Int, $after: ID, $orderBy: ReportsOrdering, $orderMode: OrderingMode) {
@@ -277,7 +317,7 @@ class Report:
             final_data = final_data + data
             while result["data"]["reports"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["reports"]["pageInfo"]["endCursor"]
-                self.opencti.log("info", "Listing Reports after " + after)
+                LOGGER.info("Listing Reports after " + after)
                 result = self.opencti.query(
                     query,
                     {
@@ -310,7 +350,7 @@ class Report:
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            self.opencti.log("info", "Reading Report {" + id + "}.")
+            LOGGER.info("Reading Report {%s}.", id)
             query = (
                 """
                 query Report($id: String!) {
@@ -365,7 +405,7 @@ class Report:
 
     """
         Check if a report already contains a thing (Stix Object or Stix Relationship)
-        
+
         :param id: the id of the Report
         :param stixObjectOrStixRelationshipId: the id of the Stix-Entity
         :return Boolean
@@ -377,13 +417,9 @@ class Report:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            self.opencti.log(
-                "info",
-                "Checking StixObjectOrStixRelationship {"
-                + stix_object_or_stix_relationship_id
-                + "} in Report {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Checking StixObjectOrStixRelationship {%s} in Report {%s}",
+                *(stix_object_or_stix_relationship_id, id),
             )
             query = """
                 query ReportContainsStixObjectOrStixRelationship($id: String!, $stixObjectOrStixRelationshipId: String!) {
@@ -399,8 +435,7 @@ class Report:
             )
             return result["data"]["reportContainsStixObjectOrStixRelationship"]
         else:
-            self.opencti.log(
-                "error",
+            LOGGER.error(
                 "[opencti_report] Missing parameters: id or stixObjectOrStixRelationshipId",
             )
 
@@ -414,6 +449,7 @@ class Report:
     def create(self, **kwargs):
         stix_id = kwargs.get("stix_id", None)
         created_by = kwargs.get("createdBy", None)
+        objects = kwargs.get("objects", None)
         object_marking = kwargs.get("objectMarking", None)
         object_label = kwargs.get("objectLabel", None)
         external_references = kwargs.get("externalReferences", None)
@@ -427,17 +463,18 @@ class Report:
         report_types = kwargs.get("report_types", None)
         published = kwargs.get("published", None)
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        granted_refs = kwargs.get("objectOrganization", None)
         update = kwargs.get("update", False)
 
         if name is not None and description is not None and published is not None:
-            self.opencti.log("info", "Creating Report {" + name + "}.")
+            LOGGER.info("Creating Report {%s}.", name)
             query = """
                 mutation ReportAdd($input: ReportAddInput) {
                     reportAdd(input: $input) {
                         id
                         standard_id
                         entity_type
-                        parent_types            
+                        parent_types
                     }
                 }
             """
@@ -449,6 +486,8 @@ class Report:
                         "createdBy": created_by,
                         "objectMarking": object_marking,
                         "objectLabel": object_label,
+                        "objectOrganization": granted_refs,
+                        "objects": objects,
                         "externalReferences": external_references,
                         "revoked": revoked,
                         "confidence": confidence,
@@ -466,9 +505,9 @@ class Report:
             )
             return self.opencti.process_multiple_fields(result["data"]["reportAdd"])
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_report] Missing parameters: name and description and published and report_class",
+            LOGGER.error(
+                "[opencti_report] "
+                "Missing parameters: name and description and published and report_class"
             )
 
     """
@@ -485,13 +524,9 @@ class Report:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            self.opencti.log(
-                "info",
-                "Adding StixObjectOrStixRelationship {"
-                + stix_object_or_stix_relationship_id
-                + "} to Report {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Adding StixObjectOrStixRelationship {%s} to Report {%s}",
+                *(stix_object_or_stix_relationship_id, id),
             )
             query = """
                mutation ReportEditRelationAdd($id: ID!, $input: StixMetaRelationshipAddInput) {
@@ -514,8 +549,7 @@ class Report:
             )
             return True
         else:
-            self.opencti.log(
-                "error",
+            LOGGER.error(
                 "[opencti_report] Missing parameters: id and stixObjectOrStixRelationshipId",
             )
             return False
@@ -534,16 +568,12 @@ class Report:
             "stixObjectOrStixRelationshipId", None
         )
         if id is not None and stix_object_or_stix_relationship_id is not None:
-            self.opencti.log(
-                "info",
-                "Removing StixObjectOrStixRelationship {"
-                + stix_object_or_stix_relationship_id
-                + "} to Report {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Removing StixObjectOrStixRelationship {%s} to Report {%s}",
+                *(stix_object_or_stix_relationship_id, id),
             )
             query = """
-               mutation ReportEditRelationDelete($id: ID!, $toId: String!, $relationship_type: String!) {
+               mutation ReportEditRelationDelete($id: ID!, $toId: StixRef!, $relationship_type: String!) {
                    reportEdit(id: $id) {
                         relationDelete(toId: $toId, relationship_type: $relationship_type) {
                             id
@@ -561,8 +591,7 @@ class Report:
             )
             return True
         else:
-            self.opencti.log(
-                "error",
+            LOGGER.error(
                 "[opencti_report] Missing parameters: id and stixObjectOrStixRelationshipId",
             )
             return False
@@ -579,19 +608,14 @@ class Report:
         extras = kwargs.get("extras", {})
         update = kwargs.get("update", False)
         if stix_object is not None:
-
-            # TODO: Compatibility with OpenCTI 3.X to be REMOVED
-            if "report_types" not in stix_object:
-                stix_object["report_types"] = (
-                    [stix_object["x_opencti_report_class"]]
-                    if "x_opencti_report_class" in stix_object
-                    else None
-                )
-            if "confidence" not in stix_object:
-                stix_object["confidence"] = (
-                    stix_object["x_opencti_source_confidence_level"]
-                    if "x_opencti_source_confidence_level" in stix_object
-                    else 0
+            # Search in extensions
+            if "x_opencti_stix_ids" not in stix_object:
+                stix_object[
+                    "x_opencti_stix_ids"
+                ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+            if "granted_refs" not in stix_object:
+                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
+                    "granted_refs", stix_object
                 )
 
             return self.create(
@@ -605,6 +629,7 @@ class Report:
                 objectLabel=extras["object_label_ids"]
                 if "object_label_ids" in extras
                 else [],
+                objects=extras["object_ids"] if "object_ids" in extras else [],
                 externalReferences=extras["external_references_ids"]
                 if "external_references_ids" in extras
                 else [],
@@ -630,7 +655,10 @@ class Report:
                 x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
                 if "x_opencti_stix_ids" in stix_object
                 else None,
+                objectOrganization=stix_object["granted_refs"]
+                if "granted_refs" in stix_object
+                else None,
                 update=update,
             )
         else:
-            self.opencti.log("error", "[opencti_report] Missing parameters: stixObject")
+            LOGGER.error("[opencti_report] Missing parameters: stixObject")
