@@ -5,6 +5,8 @@ import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
+from pycti.entities import LOGGER
+
 
 class StixCoreRelationship:
     def __init__(self, opencti):
@@ -187,6 +189,32 @@ class StixCoreRelationship:
                 ... on Incident {
                     name
                 }
+                ... on Event {
+                    name
+                    description
+                }
+                ... on Channel {
+                    name
+                    description
+                }
+                ... on Narrative {
+                    name
+                    description
+                }
+                ... on Language {
+                    name
+                }
+                ... on DataComponent {
+                    name
+                    description
+                }
+                ... on DataSource {
+                    name
+                    description
+                }
+                ... on Case {
+                    name
+                }
                 ... on StixCyberObservable {
                     observable_value
                 }
@@ -271,6 +299,32 @@ class StixCoreRelationship:
                 ... on Incident {
                     name
                 }
+                ... on Event {
+                    name
+                    description
+                }
+                ... on Channel {
+                    name
+                    description
+                }
+                ... on Narrative {
+                    name
+                    description
+                }
+                ... on Language {
+                    name
+                }
+                ... on DataComponent {
+                    name
+                    description
+                }
+                ... on DataSource {
+                    name
+                    description
+                }
+                ... on Case {
+                    name
+                }
                 ... on StixCyberObservable {
                     observable_value
                 }
@@ -299,6 +353,13 @@ class StixCoreRelationship:
                 "start_time": start_time,
                 "stop_time": stop_time,
             }
+        elif start_time is not None:
+            data = {
+                "relationship_type": relationship_type,
+                "source_ref": source_ref,
+                "target_ref": target_ref,
+                "start_time": start_time,
+            }
         else:
             data = {
                 "relationship_type": relationship_type,
@@ -326,6 +387,7 @@ class StixCoreRelationship:
 
     def list(self, **kwargs):
         element_id = kwargs.get("elementId", None)
+        element_with_target_types = kwargs.get("elementWithTargetTypes", None)
         from_id = kwargs.get("fromId", None)
         from_types = kwargs.get("fromTypes", None)
         to_id = kwargs.get("toId", None)
@@ -346,20 +408,16 @@ class StixCoreRelationship:
         if get_all:
             first = 100
 
-        self.opencti.log(
-            "info",
-            "Listing stix_core_relationships with {type: "
-            + str(relationship_type)
-            + ", from_id: "
-            + str(from_id)
-            + ", to_id: "
-            + str(to_id)
-            + "}",
+        LOGGER.info(
+            "Listing stix_core_relationships with {type: %s, element_id: %s, from_id: %s, "
+            "to_id: %s, element_with_target_types: %s, from_types: %s, to_types: %s}",
+            *(relationship_type, element_id, from_id),
+            *(to_id, element_with_target_types, from_types, to_types),
         )
         query = (
             """
-                query StixCoreRelationships($elementId: [String], $fromId: [String], $fromTypes: [String], $toId: [String], $toTypes: [String], $relationship_type: [String], $startTimeStart: DateTime, $startTimeStop: DateTime, $stopTimeStart: DateTime, $stopTimeStop: DateTime, $filters: [StixCoreRelationshipsFiltering], $first: Int, $after: ID, $orderBy: StixCoreRelationshipsOrdering, $orderMode: OrderingMode) {
-                    stixCoreRelationships(elementId: $elementId, fromId: $fromId, fromTypes: $fromTypes, toId: $toId, toTypes: $toTypes, relationship_type: $relationship_type, startTimeStart: $startTimeStart, startTimeStop: $startTimeStop, stopTimeStart: $stopTimeStart, stopTimeStop: $stopTimeStop, filters: $filters, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
+                query StixCoreRelationships($elementId: [String], $elementWithTargetTypes: [String], $fromId: [String], $fromTypes: [String], $toId: [String], $toTypes: [String], $relationship_type: [String], $startTimeStart: DateTime, $startTimeStop: DateTime, $stopTimeStart: DateTime, $stopTimeStop: DateTime, $filters: [StixCoreRelationshipsFiltering], $first: Int, $after: ID, $orderBy: StixCoreRelationshipsOrdering, $orderMode: OrderingMode) {
+                    stixCoreRelationships(elementId: $elementId, elementWithTargetTypes: $elementWithTargetTypes, fromId: $fromId, fromTypes: $fromTypes, toId: $toId, toTypes: $toTypes, relationship_type: $relationship_type, startTimeStart: $startTimeStart, startTimeStop: $startTimeStop, stopTimeStart: $stopTimeStart, stopTimeStop: $stopTimeStop, filters: $filters, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                         edges {
                             node {
                                 """
@@ -382,6 +440,7 @@ class StixCoreRelationship:
             query,
             {
                 "elementId": element_id,
+                "elementWithTargetTypes": element_with_target_types,
                 "fromId": from_id,
                 "fromTypes": from_types,
                 "toId": to_id,
@@ -406,11 +465,12 @@ class StixCoreRelationship:
             final_data = final_data + data
             while result["data"]["stixCoreRelationships"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["stixCoreRelationships"]["pageInfo"]["endCursor"]
-                self.opencti.log("info", "Listing StixCoreRelationships after " + after)
+                LOGGER.info("Listing StixCoreRelationships after " + after)
                 result = self.opencti.query(
                     query,
                     {
                         "elementId": element_id,
+                        "elementWithTargetTypes": element_with_target_types,
                         "fromId": from_id,
                         "fromTypes": from_types,
                         "toId": to_id,
@@ -464,7 +524,7 @@ class StixCoreRelationship:
         stop_time_stop = kwargs.get("stopTimeStop", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            self.opencti.log("info", "Reading stix_core_relationship {" + id + "}.")
+            LOGGER.info("Reading stix_core_relationship {%s}.", id)
             query = (
                 """
                     query StixCoreRelationship($id: String!) {
@@ -500,7 +560,7 @@ class StixCoreRelationship:
             else:
                 return None
         else:
-            self.opencti.log("error", "Missing parameters: id or from_id and to_id")
+            LOGGER.error("Missing parameters: id or from_id and to_id")
             return None
 
     """
@@ -528,17 +588,14 @@ class StixCoreRelationship:
         object_label = kwargs.get("objectLabel", None)
         external_references = kwargs.get("externalReferences", None)
         kill_chain_phases = kwargs.get("killChainPhases", None)
+        granted_refs = kwargs.get("objectOrganization", None)
         update = kwargs.get("update", False)
 
-        self.opencti.log(
-            "info",
-            "Creating stix_core_relationship '"
-            + relationship_type
-            + "' {"
-            + from_id
-            + ", "
-            + to_id
-            + "}.",
+        LOGGER.info(
+            "Creating stix_core_relationship '%s' {%s, %s}.",
+            relationship_type,
+            from_id,
+            to_id,
         )
         query = """
                 mutation StixCoreRelationshipAdd($input: StixCoreRelationshipAddInput!) {
@@ -569,6 +626,7 @@ class StixCoreRelationship:
                     "createdBy": created_by,
                     "objectMarking": object_marking,
                     "objectLabel": object_label,
+                    "objectOrganization": granted_refs,
                     "externalReferences": external_references,
                     "killChainPhases": kill_chain_phases,
                     "update": update,
@@ -591,10 +649,7 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         input = kwargs.get("input", None)
         if id is not None and input is not None:
-            self.opencti.log(
-                "info",
-                "Updating stix_core_relationship {" + id + "}",
-            )
+            LOGGER.info("Updating stix_core_relationship {%s}.", id)
             query = """
                     mutation StixCoreRelationshipEdit($id: ID!, $input: [EditInput]!) {
                         stixCoreRelationshipEdit(id: $id) {
@@ -617,8 +672,7 @@ class StixCoreRelationship:
                 result["data"]["stixCoreRelationshipEdit"]["fieldPatch"]
             )
         else:
-            self.opencti.log(
-                "error",
+            LOGGER.error(
                 "[opencti_stix_core_relationship] Missing parameters: id and key and value",
             )
             return None
@@ -633,7 +687,7 @@ class StixCoreRelationship:
     def delete(self, **kwargs):
         id = kwargs.get("id", None)
         if id is not None:
-            self.opencti.log("info", "Deleting stix_core_relationship {" + id + "}.")
+            LOGGER.info("Deleting stix_core_relationship {%s}.", id)
             query = """
                 mutation StixCoreRelationshipEdit($id: ID!) {
                     stixCoreRelationshipEdit(id: $id) {
@@ -643,9 +697,7 @@ class StixCoreRelationship:
             """
             self.opencti.query(query, {"id": id})
         else:
-            self.opencti.log(
-                "error", "[opencti_stix_core_relationship] Missing parameters: id"
-            )
+            LOGGER.error("[opencti_stix_core_relationship] Missing parameters: id")
             return None
 
     """
@@ -682,20 +734,14 @@ class StixCoreRelationship:
                 id=id, customAttributes=custom_attributes
             )
             if stix_core_relationship is None:
-                self.opencti.log(
-                    "error", "Cannot add Marking-Definition, entity not found"
-                )
+                LOGGER.error("Cannot add Marking-Definition, entity not found")
                 return False
             if marking_definition_id in stix_core_relationship["objectMarkingIds"]:
                 return True
             else:
-                self.opencti.log(
-                    "info",
-                    "Adding Marking-Definition {"
-                    + marking_definition_id
-                    + "} to Stix-Domain-Object {"
-                    + id
-                    + "}",
+                LOGGER.info(
+                    "Adding Marking-Definition {%s} to Stix-Domain-Object {%s}",
+                    *(marking_definition_id, id),
                 )
                 query = """
                    mutation StixCoreRelationshipAddRelation($id: ID!, $input: StixMetaRelationshipAddInput) {
@@ -718,9 +764,7 @@ class StixCoreRelationship:
                 )
                 return True
         else:
-            self.opencti.log(
-                "error", "Missing parameters: id and marking_definition_id"
-            )
+            LOGGER.error("Missing parameters: id and marking_definition_id")
             return False
 
     """
@@ -735,16 +779,12 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         marking_definition_id = kwargs.get("marking_definition_id", None)
         if id is not None and marking_definition_id is not None:
-            self.opencti.log(
-                "info",
-                "Removing Marking-Definition {"
-                + marking_definition_id
-                + "} from stix_core_relationship {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Removing Marking-Definition {%s} from stix_core_relationship {%s}",
+                *(marking_definition_id, id),
             )
             query = """
-               mutation StixCoreRelationshipRemoveRelation($id: ID!, $toId: String!, $relationship_type: String!) {
+               mutation StixCoreRelationshipRemoveRelation($id: ID!, $toId: StixRef!, $relationship_type: String!) {
                    stixCoreRelationshipEdit(id: $id) {
                         relationDelete(toId: $toId, relationship_type: $relationship_type) {
                             id
@@ -762,7 +802,7 @@ class StixCoreRelationship:
             )
             return True
         else:
-            self.opencti.log("error", "Missing parameters: id and label_id")
+            LOGGER.error("Missing parameters: id and label_id")
             return False
 
     """
@@ -787,13 +827,8 @@ class StixCoreRelationship:
                 label = self.opencti.label.create(value=label_name)
                 label_id = label["id"]
         if id is not None and label_id is not None:
-            self.opencti.log(
-                "info",
-                "Adding label {"
-                + label_id
-                + "} to stix-core-relationship {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Adding label {%s} to stix-core-relationship {%s}", label_id, id
             )
             query = """
                mutation StixCoreRelationshipAddRelation($id: ID!, $input: StixMetaRelationshipAddInput) {
@@ -816,7 +851,7 @@ class StixCoreRelationship:
             )
             return True
         else:
-            self.opencti.log("error", "Missing parameters: id and label_id")
+            LOGGER.error("Missing parameters: id and label_id")
             return False
 
     """
@@ -831,13 +866,9 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         external_reference_id = kwargs.get("external_reference_id", None)
         if id is not None and external_reference_id is not None:
-            self.opencti.log(
-                "info",
-                "Adding External-Reference {"
-                + external_reference_id
-                + "} to stix-core-relationship {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Adding External-Reference {%s} to stix-core-relationship {%s}",
+                *(external_reference_id, id),
             )
             query = """
                mutation StixCoreRelationshipEditRelationAdd($id: ID!, $input: StixMetaRelationshipAddInput) {
@@ -860,9 +891,7 @@ class StixCoreRelationship:
             )
             return True
         else:
-            self.opencti.log(
-                "error", "Missing parameters: id and external_reference_id"
-            )
+            LOGGER.error("Missing parameters: id and external_reference_id")
             return False
 
     """
@@ -877,16 +906,12 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         external_reference_id = kwargs.get("external_reference_id", None)
         if id is not None and external_reference_id is not None:
-            self.opencti.log(
-                "info",
-                "Removing External-Reference {"
-                + external_reference_id
-                + "} from stix_core_relationship {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Removing External-Reference {%s} from stix_core_relationship {%s}",
+                *(external_reference_id, id),
             )
             query = """
-               mutation StixCoreRelationshipRemoveRelation($id: ID!, $toId: String!, $relationship_type: String!) {
+               mutation StixCoreRelationshipRemoveRelation($id: ID!, $toId: StixRef!, $relationship_type: String!) {
                    stixCoreRelationshipEdit(id: $id) {
                         relationDelete(toId: $toId, relationship_type: $relationship_type) {
                             id
@@ -904,7 +929,7 @@ class StixCoreRelationship:
             )
             return True
         else:
-            self.opencti.log("error", "Missing parameters: id and label_id")
+            LOGGER.error("Missing parameters: id and label_id")
             return False
 
     """
@@ -919,13 +944,9 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         kill_chain_phase_id = kwargs.get("kill_chain_phase_id", None)
         if id is not None and kill_chain_phase_id is not None:
-            self.opencti.log(
-                "info",
-                "Adding Kill-Chain-Phase {"
-                + kill_chain_phase_id
-                + "} to stix-core-relationship {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Adding Kill-Chain-Phase {%s} to stix-core-relationship {%s}",
+                *(kill_chain_phase_id, id),
             )
             query = """
                mutation StixCoreRelationshipAddRelation($id: ID!, $input: StixMetaRelationshipAddInput) {
@@ -948,8 +969,7 @@ class StixCoreRelationship:
             )
             return True
         else:
-            self.opencti.log(
-                "error",
+            LOGGER.error(
                 "[opencti_stix_core_relationship] Missing parameters: id and kill_chain_phase_id",
             )
             return False
@@ -966,16 +986,12 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         kill_chain_phase_id = kwargs.get("kill_chain_phase_id", None)
         if id is not None and kill_chain_phase_id is not None:
-            self.opencti.log(
-                "info",
-                "Removing Kill-Chain-Phase {"
-                + kill_chain_phase_id
-                + "} from stix_core_relationship {"
-                + id
-                + "}",
+            LOGGER.info(
+                "Removing Kill-Chain-Phase {%s} from stix_core_relationship {%s}",
+                *(kill_chain_phase_id, id),
             )
             query = """
-               mutation StixCoreRelationshipRemoveRelation($id: ID!, $toId: String!, $relationship_type: String!) {
+               mutation StixCoreRelationshipRemoveRelation($id: ID!, $toId: StixRef!, $relationship_type: String!) {
                    stixCoreRelationshipEdit(id: $id) {
                         relationDelete(toId: $toId, relationship_type: $relationship_type) {
                             id
@@ -993,9 +1009,8 @@ class StixCoreRelationship:
             )
             return True
         else:
-            self.opencti.log(
-                "error",
-                "[stix_core_relationship] Missing parameters: id and kill_chain_phase_id",
+            LOGGER.error(
+                "[stix_core_relationship] Missing parameters: id and kill_chain_phase_id"
             )
             return False
 
@@ -1011,13 +1026,9 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         identity_id = kwargs.get("identity_id", None)
         if id is not None:
-            self.opencti.log(
-                "info",
-                "Updating author of stix_core_relationship {"
-                + id
-                + "} with Identity {"
-                + str(identity_id)
-                + "}",
+            LOGGER.info(
+                "Updating author of stix_core_relationship {%s} with Identity {%s}",
+                *(id, identity_id),
             )
             custom_attributes = """
                 id
@@ -1046,7 +1057,7 @@ class StixCoreRelationship:
             stix_domain_object = self.read(id=id, customAttributes=custom_attributes)
             if stix_domain_object["createdBy"] is not None:
                 query = """
-                    mutation StixCoreRelationshipEdit($id: ID!, $toId: String! $relationship_type: String!) {
+                    mutation StixCoreRelationshipEdit($id: ID!, $toId: StixRef! $relationship_type: String!) {
                         stixCoreRelationshipEdit(id: $id) {
                             relationDelete(toId: $toId, relationship_type: $relationship_type) {
                                 id
@@ -1082,7 +1093,7 @@ class StixCoreRelationship:
                 }
                 self.opencti.query(query, variables)
         else:
-            self.opencti.log("error", "Missing parameters: id")
+            LOGGER.error("Missing parameters: id")
             return False
 
     """
@@ -1098,6 +1109,12 @@ class StixCoreRelationship:
         update = kwargs.get("update", False)
         default_date = kwargs.get("defaultDate", False)
         if stix_relation is not None:
+            # Search in extensions
+            if "granted_refs" not in stix_relation:
+                stix_relation["granted_refs"] = self.opencti.get_attribute_in_extension(
+                    "granted_refs", stix_relation
+                )
+
             source_ref = stix_relation["source_ref"]
             target_ref = stix_relation["target_ref"]
             return self.create(
@@ -1144,9 +1161,10 @@ class StixCoreRelationship:
                 killChainPhases=extras["kill_chain_phases_ids"]
                 if "kill_chain_phases_ids" in extras
                 else None,
+                objectOrganization=stix_relation["granted_refs"]
+                if "granted_refs" in stix_relation
+                else None,
                 update=update,
             )
         else:
-            self.opencti.log(
-                "error", "[opencti_attack_pattern] Missing parameters: stixObject"
-            )
+            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
