@@ -1,5 +1,7 @@
 # coding: utf-8
 
+from pycti.entities import LOGGER
+
 
 class StixObjectOrStixRelationship:
     def __init__(self, opencti):
@@ -129,6 +131,8 @@ class StixObjectOrStixRelationship:
                 attribute_abstract
                 content
                 authors
+                note_types
+                likelihood
             }
             ... on ObservedData {
                 first_observed
@@ -324,6 +328,35 @@ class StixObjectOrStixRelationship:
                 last_seen
                 objective
             }
+            ... on Event {
+                name
+                description
+              }
+             ... on Channel {
+                name
+                description
+              }
+             ... on Narrative {
+                name
+                description
+              }
+             ... on Language {
+                name
+            }
+            ... on DataComponent {
+                name
+                description
+            }
+            ... on DataSource {
+                name
+                description
+            }
+            ... on Case {
+                name
+            }            
+            ... on StixCyberObservable {
+                observable_value
+            }
             ... on StixCoreRelationship {
                 createdBy {
                     ... on Identity {
@@ -478,9 +511,7 @@ class StixObjectOrStixRelationship:
         id = kwargs.get("id", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            self.opencti.log(
-                "info", "Reading StixObjectOrStixRelationship {" + id + "}."
-            )
+            LOGGER.info("Reading StixObjectOrStixRelationship {%s}.", id)
             query = (
                 """
                 query StixObjectOrStixRelationship($id: String!) {
@@ -501,5 +532,5 @@ class StixObjectOrStixRelationship:
                 result["data"]["stixObjectOrStixRelationship"]
             )
         else:
-            self.opencti.log("error", "Missing parameters: id")
+            LOGGER.error("Missing parameters: id")
             return None
