@@ -6,6 +6,8 @@ from typing import Union
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
+from pycti.entities import LOGGER
+
 
 class ThreatActor:
     """Main ThreatActor class for OpenCTI
@@ -180,9 +182,7 @@ class ThreatActor:
         if get_all:
             first = 500
 
-        self.opencti.log(
-            "info", "Listing Threat-Actors with filters " + json.dumps(filters) + "."
-        )
+        LOGGER.info("Listing Threat-Actors with filters %s.", json.dumps(filters))
         query = (
             """
             query ThreatActors($filters: [ThreatActorsFiltering], $search: String, $first: Int, $after: ID, $orderBy: ThreatActorsOrdering, $orderMode: OrderingMode) {
@@ -238,7 +238,7 @@ class ThreatActor:
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            self.opencti.log("info", "Reading Threat-Actor {" + id + "}.")
+            LOGGER.info("Reading Threat-Actor {%s}.", id)
             query = (
                 """
                 query ThreatActor($id: String!) {
@@ -263,9 +263,7 @@ class ThreatActor:
             else:
                 return None
         else:
-            self.opencti.log(
-                "error", "[opencti_threat_actor] Missing parameters: id or filters"
-            )
+            LOGGER.error("[opencti_threat_actor] Missing parameters: id or filters")
             return None
 
     def create(self, **kwargs):
@@ -328,10 +326,11 @@ class ThreatActor:
         secondary_motivations = kwargs.get("secondary_motivations", None)
         personal_motivations = kwargs.get("personal_motivations", None)
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        granted_refs = kwargs.get("objectOrganization", None)
         update = kwargs.get("update", False)
 
         if name is not None and description is not None:
-            self.opencti.log("info", "Creating Threat-Actor {" + name + "}.")
+            LOGGER.info("Creating Threat-Actor {%s}.", name)
             query = """
                 mutation ThreatActorAdd($input: ThreatActorAddInput) {
                     threatActorAdd(input: $input) {
@@ -350,6 +349,7 @@ class ThreatActor:
                         "createdBy": created_by,
                         "objectMarking": object_marking,
                         "objectLabel": object_label,
+                        "objectOrganization": granted_refs,
                         "externalReferences": external_references,
                         "revoked": revoked,
                         "confidence": confidence,
@@ -377,9 +377,8 @@ class ThreatActor:
                 result["data"]["threatActorAdd"]
             )
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_threat_actor] Missing parameters: name and description",
+            LOGGER.error(
+                "[opencti_threat_actor] Missing parameters: name and description"
             )
 
     """
@@ -394,12 +393,15 @@ class ThreatActor:
         extras = kwargs.get("extras", {})
         update = kwargs.get("update", False)
         if stix_object is not None:
-
             # Search in extensions
             if "x_opencti_stix_ids" not in stix_object:
                 stix_object[
                     "x_opencti_stix_ids"
                 ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+            if "granted_refs" not in stix_object:
+                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
+                    "granted_refs", stix_object
+                )
 
             return self.create(
                 stix_id=stix_object["id"],
@@ -457,9 +459,10 @@ class ThreatActor:
                 x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
                 if "x_opencti_stix_ids" in stix_object
                 else None,
+                objectOrganization=stix_object["granted_refs"]
+                if "granted_refs" in stix_object
+                else None,
                 update=update,
             )
         else:
-            self.opencti.log(
-                "error", "[opencti_attack_pattern] Missing parameters: stixObject"
-            )
+            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
