@@ -30,6 +30,10 @@ class StixCyberObservableTypes(Enum):
     CRYPTOCURRENCY_WALLET = "Cryptocurrency-Wallet"
     TEXT = "Text"
     USER_AGENT = "User-Agent"
+    BANK_ACCOUNT = "Bank-Account"
+    PHONE_NUMBER = "Phone-Number"
+    PAYMENT_CARD = "Payment-Card"
+    MEDIA_CONTENT = "Media-Content"
     SIMPLE_OBSERVABLE = "Simple-Observable"
 
     @classmethod
@@ -51,9 +55,10 @@ class IdentityTypes(Enum):
 
 
 class LocationTypes(Enum):
-    CITY = "City"
-    COUNTRY = "Country"
     REGION = "Region"
+    COUNTRY = "Country"
+    ADMINISTRATIVE_AREA = "Administrative-Area"
+    CITY = "City"
     POSITION = "Position"
 
     @classmethod
@@ -67,6 +72,8 @@ class ContainerTypes(Enum):
     OBSERVED_DATA = "Observed-Data"
     OPINION = "Opinion"
     REPORT = "Report"
+    GROUPING = "Grouping"
+    CASE = "Case"
 
     @classmethod
     def has_value(cls, value):
@@ -99,7 +106,7 @@ class MultipleStixCyberObservableRelationship(Enum):
     CHILD = "child"
     BODY_MULTIPART = "body-multipart"
     VALUES = "values"
-    LINKED = "x_opencti_linked-to"
+    LINKED = "linked-to"
 
     @classmethod
     def has_value(cls, value):
